import csv
from datetime import datetime
from abc import ABCMeta, abstractmethod
"""
Module Description:

This module defines a hierarchy of fruit classes (subclasses: Apple, Cherry, Peach) that represent various fruits
planted in an orchard. It includes functionality for checking fruit availability, storing and loading
fruit information to and from CSV files, and retrieving specific fruit instances based on type number
and variety.

Please see the docstrings within each function and class for more detailed information.
"""


class Fruit(metaclass=ABCMeta):
    """
    Represents an abstract base class for various fruits planted in orchard.
    It has three subclasses: Apple, Cherry and Peach

    Note:
    This is an abstract base class, and subclasses are required to implement get_type_num, get_available_season,
         and get_type methods.
    The `variety` parameter must be one of the varieties stored in the `season_dict` of the subclass.
    No two fruit classes in the orchard are allowed to have same type number and variety.
    """

    def __init__(self, variety, size, sweet, sour, taste, price, use):
        """
        Initializes an instance of Fruit.

        Parameters:
        - variety (str): The variety of the fruit.
        - size (str): The size of the fruit. e.g: medium, big
        - sweet (float): The sweetness of the fruit e.g. very sweet
        - sour (float): The sourness of the fruit. e.g. less sour
        - taste (str): The taste of the fruit, e.g., 'crunchy'.
        - price (float): The price of the fruit. e.g 2.99
        - use (str): The intended use of the fruit. e.g: "pie"
        """
        self.variety = variety
        self.size = size
        self.sweet = sweet
        self.sour = sour
        self.taste = taste
        self.__price = price
        self.use = use

    def describe(self):
        """
        Returns a brief statement describing the fruit, including size, sourness, sweetness, taste, and use.
        """
        describe_sentence = (f"{self.size}, {self.sour}, {self.sweet}, {self.taste}." +
                             f"It is good for {self.use}")
        return describe_sentence

    # def __str__(self):
    #     return Fruit.describe(self)

    def get_price(self):
        """
        Returns the price of the fruit.
        """
        return self.__price


    def set_price(self, price):
        """
        Sets the price of the fruit.
        """
        self.__price = price


    @abstractmethod
    def get_type_num(self):
        """
        Abstract method, returns the type number of the fruit.
        """
        pass

    @abstractmethod
    def get_available_season(self, date):
        """
        Abstract method, returns the available season of the fruit for the given date.
        """
        pass

    @abstractmethod
    def get_type(self):
        """
        Abstract method, returns the type of the fruit.
        """
        pass

class Apple(Fruit):
    """
    Represents an Apple, a specific subclass of Fruit.

    Class Attributes:
    - type_num (int): The unique type number assigned to all instances of Apple. always set to 1
    - type (str): The type of fruit, always set to "apple".
    - season_dict (dict): A dictionary correlating the seasons with the variety of apples
      allowed for cultivation in the orchard.
    """

    type_num = 1
    type = "apple"

    season_dict = {"Ambrosia": [9,10],
                   "Gala": [8,9,10],
                   "Honeycrisp": [9,10]}

    def __init__(self, variety, size, sweet, sour, taste, price, use):
        """
        Initializes an instance of Apple.
        """
        Fruit.__init__(self, variety, size, sweet, sour, taste, price, use)

    def get_type_num(self):
        """
        Returns the unique type number of the Apple. always 1
        """
        return Apple.type_num

    def get_type(self):
        """
        Returns the type of the Apple, always "apple".
        """
        return Apple.type

    def get_available_season(self, date):
        """
        Returns True if the Apple variety is available in the given date's season, False otherwise.
        Meanwhile, print at the given season, the Apple variety is in season, out of season, or
        season has not arrived yet.
        """
        season_list = Apple.season_dict[self.variety]
        if date in season_list:
            print(f"{self.variety} Apple is available now")
            return True
        elif date < min(season_list):
            print(f"Sorry, {self.variety} Apple is not ripe now.")
            return False
        else:
            print(f"Sorry, {self.variety} Apple season has ended.")
            return False


class Cherry(Fruit):
    """
    Represents an Cherry, a specific subclass of Fruit.

    Class Attributes:
    - type_num (int): The unique type number assigned to all instances of Cherry. always set to 2
    - type (str): The type of fruit, always set to "cherry".
    - season_dict (dict): A dictionary correlating the seasons with the variety of cherry
      allowed for cultivation in the orchard.
    """
    type_num = 2
    type = "cherry"

    season_dict = {"Lapins": [6,7],
                   "Sweetheart": [6,7],
                   "Skeena": [7,8]}

    def __init__(self, variety, size, sweet, sour, taste, price, use):
        """
        Initializes an instance of Cherry.
        """
        Fruit.__init__(self, variety, size, sweet, sour, taste, price, use)

    def get_type_num(self):
        """
        Returns the unique type number of the Cherry. always 2
        """
        return Cherry.type_num

    def get_type(self):
        """
        Returns the type of the Apple, always "cherry".
        """
        return Cherry.type

    def get_available_season(self, date):
        """
        Returns True if the Cherry variety is available in the given date's season, False otherwise.
        Meanwhile, print at the given season, the Cherry variety is in season, out of season, or
        season has not arrived yet.
        """

        season_list = Cherry.season_dict[self.variety]
        if date in season_list:
            print(f"{self.variety} Cherry is available now")
            return True
        elif date < min(season_list):
            print(f"Sorry, {self.variety} Cherry is not ripe now.")
            return False
        else:
            print(f"Sorry, {self.variety} Cherry season has ended.")
            return False



class Peach(Fruit):
    """
    Represents an Peach, a specific subclass of Fruit.

    Class Attributes:
    - type_num (int): The unique type number assigned to all instances of Peach. always set to 1
    - type (str): The type of fruit, always set to "peach".
    - season_dict (dict): A dictionary correlating the seasons with the variety of peaches
      allowed for cultivation in the orchard.
    """

    type_num = 3
    type = "peach"

    season_dict = {"Redhaven": [7, 8],
                   "Elberta": [8, 9],
                   "Cresthaven": [8, 9]}

    def __init__(self, variety, size, sweet, sour, taste, price, use):
        """
        Initializes an instance of Apple.
        """
        Fruit.__init__(self, variety, size, sweet, sour, taste, price, use)

    def get_type_num(self):
        """
        Returns the unique type number of the Peach. always 3
        """
        return Peach.type_num

    def get_type(self):
        """
        Returns the type of the Apple, always "peach".
        """
        return Peach.type

    def get_available_season(self, date):
        """
        Returns True if the Peach variety is available in the given date's season, False otherwise.
        Meanwhile, print at the given season, the Peach variety is in season, out of season, or
        season has not arrived yet.
        """
        season_list = Peach.season_dict[self.variety]
        if date in season_list:
            print(f"{self.variety} Peach is available now")
            return True
        elif date < min(season_list):
            print(f"Sorry, {self.variety} Peach is not ripe now.")
            return False
        else:
            print(f"Sorry, {self.variety} Peach season has ended.")
            return False


def datetime_transfer(date):
    """
    Convert 'MM-DD' date string to month (MM).
    """
    try:
        if isinstance(date, str):
            date = datetime.strptime(date, '%m-%d')
        month = date.month
        return month
    except:
        print("Invalid datetime")


def fruit_available_check(date, fruit):
    """Check if the input fruit is in season for the given date."""
    month = datetime_transfer(date)
    return fruit.get_available_season(month)


def available_season_fruit(date,fruit_list):
    """Check which fruits are in season for the given date and return a list of fruit."""
    month = datetime_transfer(date)
    available_list = []
    for f in fruit_list:
        variety = f.variety
        season_list = f.season_dict[variety]
        if month in season_list:
            available_list.append(f)
    return available_list


def file_store(file_path, store_data):
    """Store data in a CSV file."""
    with open(file_path, mode='w', newline='') as file:
        writer = csv.writer(file)
        writer.writerows(store_data)


def fruit_information_store(fruit_list, csv_file_path):
    """Store fruit information in a CSV file for later retrieval."""
    #csv_file_path = "fruits.csv"

    fruit_info_list = []
    for fruit in fruit_list:
        if fruit is not None:
            type_num = fruit.get_type_num()
            variety = fruit.variety
            size = fruit.size
            sweet = fruit.sweet
            sour = fruit.sour
            taste = fruit.taste
            price = round(fruit.get_price(), 2)
            use = fruit.use

            fruit_info = tuple([type_num, variety, size, sweet, sour, taste, price, use])
            fruit_info_list.append(fruit_info)

    file_store(csv_file_path, fruit_info_list)


def file_load(file_path):
    """Helper method: Load data from a file. Returns a list of file rows."""
    file_rows = []
    try:
        with open(file_path, "r") as infile:
            for line in infile:
                line = line.strip(" \n")
                fields = line.split(",")
                for i in range(0, len(fields)):
                    fields[i] = fields[i].strip()
                file_rows.append(fields)

        return file_rows
    except FileNotFoundError:
        print(f"There is no file {file_path}")


def fruit_class_load(fruit_file):
    """Load fruit information from a file and return a list of Fruit instances."""
    try:
        file_rows = file_load(fruit_file)
        fruit_list = []

        if file_rows == [['']]:
            # check if the file exist but empty
            return fruit_list

        for row in file_rows:
            fruit = None
            # title_list = ["type_num", "variety", "size", "sweet", "sour", "taste", "price", "use"]
            type_num = int(row[0])
            variety = row[1]
            size = row[2]
            sweet = row[3]
            sour = row[4]
            taste = row[5]
            price = float(row[6])
            use = row[7]

            if type_num == 1:
                fruit = Apple(variety, size, sweet, sour, taste, price, use)
            elif type_num == 2:
                fruit = Cherry(variety, size, sweet, sour, taste, price, use)
            elif type_num == 3:
                fruit = Peach(variety, size, sweet, sour, taste, price, use)
            else:
                print("These is not a valid production type num, please check your file content")
            fruit_list.append(fruit)
        return fruit_list

    except:
        # check if the file is not exist
        print("None file exist")


def get_fruit(fruit_type_num, variety, fruit_list):
    """Retrieve a unique fruit class based on type number and variety from the given fruit list."""
    for fruit in fruit_list:
        if fruit_type_num == fruit.get_type_num() and variety == fruit.variety:
            return fruit


def add_fruit(fruit_type_num, variety, size, sweet, sour, taste, price, use, fruit_file):
    """
    Add a new type of fruit record to the list of fruits. The new fruit should not have same type_num
    and variety with existed fruits.
    """
    # check there is no same fruit record at first
    fruit_list = fruit_class_load(fruit_file)
    fruit_created = None
    if get_fruit(fruit_type_num, variety, fruit_list) is None:
        if fruit_type_num == Apple.type_num and variety in list(Apple.season_dict.keys()):
            fruit_created = Apple(variety, size, sweet, sour, taste, price, use)
        elif fruit_type_num == Cherry.type_num and variety in list(Cherry.season_dict.keys()):
            fruit_created = Cherry(variety, size, sweet, sour, taste, price, use)
        elif fruit_type_num == Peach.type_num and variety in list(Peach.season_dict.keys()):
            fruit_created = Peach(variety, size, sweet, sour, taste, price, use)
        else:
            print("Invalid input, please enter fruit information with correct type_num and variety")
        fruit_list.append(fruit_created)
        fruit_information_store(fruit_list, fruit_file)
    else:
        print("Fail to add new fruit, the input fruit is already exist")


def remove_fruit(fruit_type_num, variety, fruit_file):
    """
    Remove a fruit record from the list of fruits based on the type number and variety.
    """
    try:
        fruit_list = fruit_class_load(fruit_file)
    except ValueError:
        fruit_list = []

    fruit_len_prev = len(fruit_list)
    print(len(fruit_list))
    for i in range(0, len(fruit_list)):
        fruit = fruit_list[i]
        if fruit.get_type_num() == fruit_type_num and fruit.variety == variety:
            del fruit_list[i]
            break
    if fruit_len_prev == len(fruit_list):
        print("Fail to remove fruit, this fruit is not exist")
    fruit_information_store(fruit_list, fruit_file)













































import pandas as pd
import matplotlib.pyplot as plt


class HistoricalPlotter:
    """
    A class for plotting historical data related to inventory management and remaining capacity.
    """

    def __init__(self, remaining_productivity_file, inventory_file):
        """
        Initialize a HistoricalPlotter instance.

        Parameters:
        - remaining_productivity_file (str): The file path for the remaining productivity data.
        - inventory_file (str): The file path for the inventory data.
        """
        self.inventory_file = inventory_file
        self.remaining_productivity_file = remaining_productivity_file

    def plot_historical_data(self, fruit_category, file_type):
        """
        Plot historical data based on the specified fruit category and file type.

        Parameters:
        - fruit_category (int): The category of fruit (1-Apple, 2-Cherry, 3-Peach).
        - file_type (int): The type of file to use (1 for inventory, 2 for remaining capacity).
        """
        if file_type == 1:
            data = pd.read_csv(self.inventory_file)
            title_file = "Inventory Management"
        else:
            data = pd.read_csv(self.remaining_productivity_file)
            title_file = "Remaining Capacity"

        x = range(1, len(data) + 1)
        if fruit_category == 1:
            labellist = ["Ambrosia", "Gala", "Honeycrisp"]
            plt.stackplot(x, data.loc[:, 'Ambrosia'], data.loc[:, 'Gala'],
                          data.loc[:, 'Honeycrisp'], labels=labellist)
            title_type = "Apple"
        elif fruit_category == 2:
            labellist = ["Lapins", "Sweetheart", "Skeena"]
            plt.stackplot(x, data.loc[:, "Lapins"], data.loc[:, "Sweetheart"],
                          data.loc[:, "Skeena"], labels=labellist)
            title_type = "Cherry"
        else:
            labellist = ["Redhaven", "Elberta", "Cresthaven"]
            plt.stackplot(x, data.loc[:, "Redhaven"], data.loc[:, "Elberta"],
                          data.loc[:, "Cresthaven"], labels=labellist)
            title_type = "Peach"
        title = f"{title_file} - {title_type}"
        plt.title(title)
        plt.xlabel('Index')
        plt.ylabel('Quantity')
        plt.legend()
        plt.show()
        return title


class InventoryManagement:
    """
    A class for managing inventory, remaining capacity, and extra productivity.

    """

    def __init__(self, extra_productivity_file, remaining_productivity_file, inventory_file):
        """
        Initialize an InventoryManagement instance.

        """
        self.extra_productivity_file = extra_productivity_file
        self.remaining_productivity_file = remaining_productivity_file
        self.inventory_file = inventory_file
        self.inventory, self.remaining_productivity, self.extra_productivity = self.load_data()

    def load_data(self):
        """
        Load data from CSV files (inventory, remaining productivity, extra productivity).

        """
        columns = ["index", "Ambrosia", "Gala", "Honeycrisp", "Lapins", "Sweetheart", "Skeena", "Redhaven", "Elberta",
                   "Cresthaven"]
        try:
            inventory_data = pd.read_csv(self.inventory_file)
        except FileNotFoundError:
            # If the file doesn't exist, return an initial empty inventory
            inventory_data = pd.DataFrame(columns=columns)
            inventory_data.to_csv(self.inventory_file, index=True)

        try:
            remaining_productivity_data = pd.read_csv(self.remaining_productivity_file)
        except FileNotFoundError:
            # If the file doesn't exist, return an initial empty remaining productivity
            remaining_productivity_data = pd.DataFrame(columns=columns)
            remaining_productivity_data.to_csv(self.remaining_productivity_file, index=True)

        try:
            extra_productivity_data = pd.read_csv(self.extra_productivity_file)
        except FileNotFoundError:
            # If the file doesn't exist, return an initial empty extra productivity
            extra_productivity_data = pd.DataFrame(columns=columns)
            extra_productivity_data.to_csv(self.extra_productivity_file, index=True)

        return inventory_data, remaining_productivity_data, extra_productivity_data

    def save_data(self, newdata, filename):
        """
        Save new data to the specified CSV file.

        Parameters:
        - newdata (list): New data to be added to the DataFrame.
        - filename (str): Name of the CSV file to save the data.
        """
        if len(newdata) == 9:
            info = [1]
            info.extend(newdata)
            self.remaining_productivity.loc[len(self.remaining_productivity)] = info
            self.remaining_productivity.to_csv(self.remaining_productivity_file, index=False)
        else:
            try:
                if filename == 1:
                    self.inventory = pd.concat([self.inventory, newdata], ignore_index=True)
                    self.inventory.to_csv(self.inventory_file, index=False)
                elif filename == 2:
                    self.remaining_productivity = pd.concat([self.remaining_productivity, newdata], ignore_index=True)
                    self.remaining_productivity.to_csv(self.remaining_productivity_file, index=False)
                else:
                    self.extra_productivity = pd.concat([self.extra_productivity, newdata], ignore_index=True)
                    self.extra_productivity.to_csv(self.extra_productivity_file, index=False)

            except FileNotFoundError:
                print(f"There is no {filename} file")
            except Exception:
                print("Fail to save data")

    def total_product_estimate(self):
        """
        Estimate total production based on the number of fruit trees and update inventory and remaining productivity.
        """
        if len(self.remaining_productivity) > 0:
            raise ValueError("File is not empty, and cannot be initialized by total capacity")

        plantation_summary = {
            'type_num': [1] * 3 + [2] * 3 + [3] * 3,
            'variety': ["Ambrosia", "Gala", "Honeycrisp", "Lapins", "Sweetheart", "Skeena", "Redhaven", "Elberta",
                        "Cresthaven"],
            'total_area': [20, 0, 0, 40, 0, 0, 0, 40, 0],
        }

        plantation_summary = pd.DataFrame(plantation_summary)
        plantation_summary['avgProduct'] = [1500, 1200, 1800, 1000, 2500, 3500, 2000, 1800, 2200]
        plantation_summary['totalProd'] = plantation_summary['total_area'] * plantation_summary['avgProduct']

        pivot_df = plantation_summary.pivot_table(columns='variety', values='totalProd', fill_value=0)
        pivot_df.reset_index(drop=True, inplace=True)
        desired_order = ["Ambrosia", "Gala", "Honeycrisp", "Lapins", "Sweetheart", "Skeena", "Redhaven", "Elberta",
                         "Cresthaven"]
        pivot_df = pivot_df[desired_order]

        new_row = list(pivot_df.iloc[0, ])
        self.save_data(new_row, self.remaining_productivity_file)

        return True

    def add_inventory(self, fruit_type, number):
        """
        Add inventory of a specific fruit type.

        Parameters:
        - fruit_type (str): The variety of fruit for inventory addition.
        - number (int): The quantity of fruit to add to the inventory.
        """
        if fruit_type not in self.inventory.columns:
            raise ValueError("Fruit not found in inventory")
        else:
            if self.inventory.empty:
                current_inventory = pd.DataFrame([[0] * len(self.inventory.columns)], columns=self.inventory.columns)
            else:
                current_inventory = self.inventory.iloc[[-1]].copy()
            new_row = current_inventory
            new_row[fruit_type] += number
            new_row["index"] = new_row["index"].max() + 1
            self.save_data(new_row, 1)
            self.remove_remaining_productivity(fruit_type, number)

    def add_extra_productivity(self, fruit_type, number):
        """
        Add extra productivity of a specific fruit type.

        Parameters:
        - fruit_type (str): The variety of fruit for extra productivity addition.
        - number (int): The quantity of extra productivity to add.
        """
        if self.extra_productivity.empty:
            current_extra = pd.DataFrame([[0] * len(self.extra_productivity.columns)],
                                         columns=self.extra_productivity.columns)
        else:
            current_extra = self.extra_productivity.iloc[[-1]].copy()
        new_row = current_extra
        new_row[fruit_type] += number
        new_row["index"] = new_row["index"].max() + 1
        self.save_data(new_row, 3)

    def remove_inventory(self, fruit_type, number):
        """
        Remove inventory of a specific fruit type.

        Parameters:
        - fruit_type (str): The variety of fruit for inventory removal.
        - number (int): The quantity of fruit to remove from the inventory.
        """
        if fruit_type not in self.inventory.columns:
            raise ValueError("Fruit not found in inventory")
        else:
            if self.inventory.empty:
                print("Inventory is empty.")
                return False
            else:
                current_inventory = self.inventory.iloc[[-1]].copy()
                current_number = current_inventory.at[current_inventory.index[-1], fruit_type]
                if current_number < number:
                    print(f"Not enough {fruit_type} in inventory, only {current_number} left")
                    return False
                new_row = current_inventory
                new_row[fruit_type] -= number
                new_row["index"] = new_row["index"].max() + 1
                self.save_data(new_row, 1)
        return True

    def remove_remaining_productivity(self, fruit_type, number, direct = 0):
        """
        Remove productivity for a specific fruit type.

        Args:
            fruit_type (str): The type of fruit for which productivity needs to be removed.
            number (int): The quantity of productivity to be removed.
            direct (int): 0 when cased by sales or inventory increase, 1 when cased by other reasons

        Returns:
            None
        """
        if fruit_type not in self.remaining_productivity.columns:
            print(f"Fruit '{fruit_type}' not found.")
            return
        else:
            # check empty
            if self.remaining_productivity.empty:
                print("Please estimate capacity first.")
                return
            else:
                current_capacity = self.remaining_productivity.iloc[[-1]].copy()

                # Check if there's enough quantity to remove
                current_number = current_capacity.at[current_capacity.index[-1], fruit_type]
                new_row = current_capacity
                if current_number < number:
                    if direct == 0 :
                        new_row[fruit_type] = 0
                        print(
                            f"The product of {fruit_type} exceeds expectations and will be recorded in the extra_productivity file.")
                        extra_number = number - current_number
                        self.add_extra_productivity(fruit_type, extra_number)
                    else:
                        print("The estimated yield will be set to 0")
                        new_row[fruit_type] = 0
                else:
                    new_row[fruit_type] -= number
                # set new index number
                new_row["index"] = new_row["index"].max() + 1
                self.save_data(new_row, 2)

    def get_current_inventory(self):
        """
        Get the current inventory status.

        Returns:
            pd.DataFrame: A DataFrame representing the current inventory status.
        """
        if not self.inventory.empty:
            print(self.inventory.iloc[[-1]])
            return list(self.inventory.iloc[-1])
        else:
            print("Inventory is empty.")
            return None

    def get_remaining_productivity(self):
        """
        Get the remaining productivity status.

        Returns:
            pd.DataFrame: A DataFrame representing the current remaining productivity status.
        """
        print(self.remaining_productivity.iloc[[-1]])
        return list(self.remaining_productivity.iloc[-1])

    def get_extra_productivity(self):
        """
        Get the extra productivity status.

        Returns:
            pd.DataFrame: A DataFrame representing the current extra productivity status.
        """
        print(self.extra_productivity.iloc[[-1]])
        return list(self.extra_productivity.iloc[-1])




import src.orchardmanagementpackage.production.fruit_info as fruit_info
import src.orchardmanagementpackage.production.plantation as plantation
fruit_list = fruit_info.fruit_class_load("fruits_test.csv")
region_list = []
area = 10
## set up regions
for i in range(0,len(fruit_list)):
    fruit = fruit_list[i]
    fruit_type_num = fruit.get_type_num()
    fruit_variety = fruit.variety
    area += 10
    region = plantation.Region(i + 1, fruit_type_num, fruit_variety, area, areaType="market")
    if i >= 1:
        region = plantation.set_picking_region(region)
        if i == 1:
            region = plantation.set_marketing_region(region)
    region_list.append(region)
for i in region_list:
    print(i.get_area_type())




plantation_file = "plantations_test.csv"
plantation.region_saving(plantation_file, region_list)

df_region = plantation.region_loading(plantation_file)
print(df_region)

region_list = plantation.region_class_tranfer(df_region)
print(region_list)
for region in region_list:
    print(region)
    print(region.get_fruit(fruit_list))

r1, r2 = plantation.fruit_index_list(region_list, 1, "Ambrosia")
print(r1)
print(r2)

print(plantation.area_amount_variety(region_list, 1, "Ambrosia"))


df_summary = plantation.region_summary(fruit_list, region_list)
print(df_summary)

d = plantation.area_summary(fruit_list, region_list)
print(d)

for region in region_list:
    region.region_display(fruit_list)

region_searched = plantation.get_region(1, region_list)
print(region_searched)

# plantation.add_region(3, 2, "Lapins", 40, "pick",
#                       fruit_list, plantation_file)
# plantation.add_region(4, 2, "Lapins", 40, "pick",
#                       fruit_list, plantation_file)

print(region_list)
print(len(region_list))
print(plantation_file)
df_summary_2 = plantation.region_summary(fruit_list, region_list)
print(df_summary_2)
print(plantation.area_summary(fruit_list, region_list))




import src.orchardmanagementpackage.production.fruit_info as fruit_info

"""
This module is a test.py for fruit_info
"""


a1 = fruit_info.Apple("Ambrosia", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
p1 = fruit_info.Peach("Elberta", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
c1 = fruit_info.Cherry("Lapins", "small", "median sweet", "median sour", "soft", 2.99, "cans")

print(a1.describe())
print(c1.describe())
print(p1.describe())
"""
expect output: 
big, less sour, very sweet, crunchy.It is good for pie
small, median sour, median sweet, soft.It is good for cans
big, less sour, very sweet, crunchy.It is good for pie
"""

print(a1.get_type_num())
print(c1.get_type_num())
print(p1.get_type_num())

"""
expect output:
1
2
3
"""

print(a1.get_price())
"""
expect output:
1.2
"""

fruit_info.fruit_available_check("9-10", a1)
fruit_info.fruit_available_check("9-10", c1)
fruit_info.fruit_available_check("9-10", p1)
"""
expect output:

Ambrosia Apple is available now
Sorry, Lapins Cherry season has ended.
Elberta Peach is available now
"""
fruit_list = [a1, p1, c1]
fruit_available_list = fruit_info.available_season_fruit("9-10", fruit_list)
for f in fruit_available_list:
    print(f)
"""
big, less sour, very sweet, crunchy.It is good for pie
big, less sour, very sweet, crunchy.It is good for pie
"""

file_path = "fruits_test.csv"
fruit_info.fruit_information_store(fruit_list, file_path)
print(fruit_info.file_load(file_path))
"""
expect output:
[['1', 'Ambrosia', 'big', 'very sweet', 'less sour', 'crunchy', '1.2', 'pie'], 
['3', 'Elberta', 'big', 'very sweet', 'less sour', 'crunchy', '1.2', 'pie'], 
['2', 'Lapins', 'small', 'median sweet', 'median sour', 'soft', '2.99', 'cans']]
"""

fruit_list = fruit_info.fruit_class_load(file_path)
for i in fruit_list:
    print(i.describe())
"""
big, less sour, very sweet, crunchy.It is good for pie
big, less sour, very sweet, crunchy.It is good for pie
small, median sour, median sweet, soft.It is good for cans
"""
fruit_searched = fruit_info.get_fruit(1, "Ambrosia", fruit_list)
print(fruit_searched.describe())
"""
expect output:
big, less sour, very sweet, crunchy.It is good for pie
"""

fruit_info.add_fruit(1, "Gala", "big", "very sweet", "less sour", "crunchy", 1.2, "pie", file_path)
fruit_info.remove_fruit(1, "Gala", file_path)
fruit_info.add_fruit(1, "Ambrosia", "big", "very sweet", "less sour", "crunchy", 1.2, "pie", file_path)
"""
expect output:

(success and no response)
(success and no response)
Fail to add new fruit, the input fruit is already exist
"""









import os.path
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import unittest
import pandas as pd
import src.orchardmanagementpackage.production.fruit_info as fruit_info
import src.orchardmanagementpackage.production.plantation as plantation
from io import StringIO


class RegionTestCase(unittest.TestCase):

    def setUp(self):
        self.fruit_list = fruit_info.fruit_class_load("production_file_test/test_mutiple.csv")
        self.a1 = self.fruit_list[0]
        self.p1 = self.fruit_list[1]
        self.c1 = self.fruit_list[2]
        self.r1 = plantation.Region(1, 1, "Ambrosia", 20, "market")
        self.r2 = plantation.Region(2, 3, "Elberta", 30, "market")
        self.r3 = plantation.Region(3, 2, "Lapins", 40, "pick")
        self.r4 = plantation.Region(4, 2, "Lapins", 40, "pick")
        self.region_list = [self.r1, self.r2, self.r3, self.r4]

    def tearDown(self):
        del self.fruit_list
        del self.a1
        del self.p1
        del self.c1
        del self.r1
        del self.r2
        del self.r3
        del self.r4
        del self.region_list

    def test_get_area_type(self):
        self.assertEqual(self.r1.get_area_type(), "market")
        self.assertEqual(self.r2.get_area_type(), "market")
        self.assertEqual(self.r3.get_area_type(), "pick")
        self.assertEqual(self.r4.get_area_type(), "pick")

    def test_set_area_type(self):
        # test unchanged area_type
        self.r1.set_area_type("market")
        self.assertEqual(self.r1.get_area_type(), "market")
        self.r3.set_area_type("pick")
        self.assertEqual(self.r3.get_area_type(), "pick")

        # test changed area_type
        self.r2.set_area_type("pick")
        self.assertEqual(self.r2.get_area_type(), "pick")
        self.r4.set_area_type("market")
        self.assertEqual(self.r4.get_area_type(), "market")

    def test_get_area(self):
        self.assertEqual(self.r1.get_area(), 20)
        self.assertEqual(self.r2.get_area(), 30)
        self.assertEqual(self.r3.get_area(), 40)
        self.assertEqual(self.r4.get_area(), 40)

    def test_get_fruit(self):
        fruit1 = self.r1.get_fruit(self.fruit_list)
        fruit2 = self.r2.get_fruit(self.fruit_list)
        fruit3 = self.r3.get_fruit(self.fruit_list)
        fruit4 = self.r4.get_fruit(self.fruit_list)

        self.assertEqual(fruit1.get_type_num(),self.a1.get_type_num())
        self.assertEqual(fruit1.describe(), self.a1.describe())

        self.assertEqual(fruit2.get_type_num(), self.p1.get_type_num())
        self.assertEqual(fruit2.describe(), self.p1.describe())

        self.assertEqual(fruit3.get_type_num(), self.c1.get_type_num())
        self.assertEqual(fruit3.describe(), self.c1.describe())

        self.assertEqual(fruit4.get_type_num(), self.c1.get_type_num())
        self.assertEqual(fruit4.describe(), self.c1.describe())

    def test_region_display(self):
        # test print output
        original_stdout = sys.stdout
        sys.stdout = StringIO()
        self.r1.region_display(self.fruit_list)
        printed_output = sys.stdout.getvalue().strip()
        sys.stdout = original_stdout
        self.assertEqual(printed_output, "Region 1: Ambrosia apple. Region Type: market. Area: 20")

        original_stdout = sys.stdout
        sys.stdout = StringIO()
        self.r2.region_display(self.fruit_list)
        printed_output = sys.stdout.getvalue().strip()
        sys.stdout = original_stdout
        self.assertEqual(printed_output, "Region 2: Elberta peach. Region Type: market. Area: 30")

        original_stdout = sys.stdout
        sys.stdout = StringIO()
        self.r3.region_display(self.fruit_list)
        printed_output = sys.stdout.getvalue().strip()
        sys.stdout = original_stdout
        self.assertEqual(printed_output, "Region 3: Lapins cherry. Region Type: pick. Area: 40")

        original_stdout = sys.stdout
        sys.stdout = StringIO()
        self.r4.region_display(self.fruit_list)
        printed_output = sys.stdout.getvalue().strip()
        sys.stdout = original_stdout
        self.assertEqual(printed_output, "Region 4: Lapins cherry. Region Type: pick. Area: 40")

    def test_get_region(self):
        r1_get = plantation.get_region(1, self.region_list)
        r2_get = plantation.get_region(2, self.region_list)
        r3_get = plantation.get_region(3, self.region_list)
        r4_get = plantation.get_region(4, self.region_list)
        self.assertEqual(r1_get.regionId, self.r1.regionId)
        self.assertEqual(r1_get.fruit_type_num, self.r1.fruit_type_num)
        self.assertEqual(r1_get.variety, self.r1.variety)
        self.assertEqual(r1_get.get_area(), self.r1.get_area())
        self.assertEqual(r1_get.get_area_type(), self.r1.get_area_type())

        self.assertEqual(r2_get.regionId, self.r2.regionId)
        self.assertEqual(r2_get.fruit_type_num, self.r2.fruit_type_num)
        self.assertEqual(r2_get.variety, self.r2.variety)
        self.assertEqual(r2_get.get_area(), self.r2.get_area())
        self.assertEqual(r2_get.get_area_type(), self.r2.get_area_type())

        self.assertEqual(r3_get.regionId, self.r3.regionId)
        self.assertEqual(r3_get.fruit_type_num, self.r3.fruit_type_num)
        self.assertEqual(r3_get.variety, self.r3.variety)
        self.assertEqual(r3_get.get_area(), self.r3.get_area())
        self.assertEqual(r3_get.get_area_type(), self.r3.get_area_type())

        self.assertEqual(r4_get.regionId, self.r4.regionId)
        self.assertEqual(r4_get.fruit_type_num, self.r4.fruit_type_num)
        self.assertEqual(r4_get.variety, self.r4.variety)
        self.assertEqual(r4_get.get_area(), self.r4.get_area())
        self.assertEqual(r4_get.get_area_type(), self.r4.get_area_type())

    def test_set_picking_region(self):
        plantation.set_picking_region(self.r1)
        self.assertEqual(self.r1.get_area_type(), "pick")

        plantation.set_picking_region(self.r2)
        self.assertEqual(self.r2.get_area_type(), "pick")

        plantation.set_picking_region(self.r3)
        self.assertEqual(self.r3.get_area_type(), "pick")

        plantation.set_picking_region(self.r4)
        self.assertEqual(self.r4.get_area_type(), "pick")

    def test_set_marketing_region(self):
        plantation.set_marketing_region(self.r1)
        self.assertEqual(self.r1.get_area_type(), "market")

        plantation.set_marketing_region(self.r2)
        self.assertEqual(self.r2.get_area_type(), "market")

        plantation.set_marketing_region(self.r3)
        self.assertEqual(self.r3.get_area_type(), "market")

        plantation.set_marketing_region(self.r4)
        self.assertEqual(self.r4.get_area_type(), "market")

    def test_area_amount_variety(self):
        # test with valid fruit type_num and valid fruit variety
        area1, area2 = plantation.area_amount_variety(self.region_list, 1, "Ambrosia")
        self.assertEqual(area1, 0)
        self.assertEqual(area2, 20)

        # test with valid fruit type_num and invalid fruit variety
        area1, area2 = plantation.area_amount_variety(self.region_list, 1, "Lapin")
        self.assertEqual(area1, 0)
        self.assertEqual(area2, 0)

        # test with invalid fruit type_num and valid fruit variety
        area1, area2 = plantation.area_amount_variety(self.region_list, 6, "Lapin")
        self.assertEqual(area1, 0)
        self.assertEqual(area2, 0)

        # test with invalid fruit type_num and invalid fruit variety
        area1, area2 = plantation.area_amount_variety(self.region_list, 6, "abc")
        self.assertEqual(area1, 0)
        self.assertEqual(area2, 0)

    def test_region_summary(self):
        # test region summary with empty region_list
        region_list_1 = []
        region_list_2 = [self.r1, self.r2, self.r3]
        df1 = plantation.region_summary(self.fruit_list, region_list_1)
        self.assertEqual(df1["Lapins cherry"]["pick_area"],0)
        self.assertEqual(df1["Lapins cherry"]["market_area"], 0)
        self.assertEqual(df1["Elberta peach"]["pick_area"], 0)
        self.assertEqual(df1["Elberta peach"]["market_area"], 0)
        self.assertEqual(df1["Ambrosia apple"]["pick_area"], 0)
        self.assertEqual(df1["Ambrosia apple"]["market_area"], 0)

        self.assertEqual(df1["Lapins cherry"]["picking_regions_index"], '[]')
        self.assertEqual(df1["Lapins cherry"]["marketing_regions_index"], '[]')
        self.assertEqual(df1["Elberta peach"]["picking_regions_index"], '[]')
        self.assertEqual(df1["Elberta peach"]["marketing_regions_index"], '[]')
        self.assertEqual(df1["Ambrosia apple"]["picking_regions_index"], '[]')
        self.assertEqual(df1["Ambrosia apple"]["marketing_regions_index"], '[]')

        # test region summary with empty fruit_list
        fruit_list_1 = []
        df2 = plantation.region_summary(fruit_list_1, region_list_2)
        self.assertTrue(df2.empty)

        # test region summary with empty fruit_list and empty region_list
        df3 = plantation.region_summary(fruit_list_1, region_list_1)
        self.assertTrue(df3.empty)

        # test region summary with fruit_list and empty region_list
        df4 = plantation.region_summary(self.fruit_list, self.region_list)
        self.assertEqual(df4["Lapins cherry"]["pick_area"], 80)
        self.assertEqual(df4["Lapins cherry"]["market_area"], 0)
        self.assertEqual(df4["Elberta peach"]["pick_area"], 0)
        self.assertEqual(df4["Elberta peach"]["market_area"], 30)
        self.assertEqual(df4["Ambrosia apple"]["pick_area"], 0)
        self.assertEqual(df4["Ambrosia apple"]["market_area"], 20)

        self.assertEqual(df4["Lapins cherry"]["picking_regions_index"], '[3, 4]')
        self.assertEqual(df4["Lapins cherry"]["marketing_regions_index"], '[None, None]')
        self.assertEqual(df4["Elberta peach"]["picking_regions_index"], '[None]')
        self.assertEqual(df4["Elberta peach"]["marketing_regions_index"], '[2]')
        self.assertEqual(df4["Ambrosia apple"]["picking_regions_index"], '[None]')
        self.assertEqual(df4["Ambrosia apple"]["marketing_regions_index"], '[1]')

    def test_area_summary(self):
        df1 = plantation.area_summary(self.fruit_list, self.region_list)
        self.assertEqual(df1["picking_area"]["1 Ambrosia"], 0)
        self.assertEqual(df1["picking_area"]["3 Elberta"], 0)
        self.assertEqual(df1["picking_area"]["2 Lapins"], 80)
        self.assertEqual(df1["marketing_area"]["1 Ambrosia"], 20)
        self.assertEqual(df1["marketing_area"]["3 Elberta"], 30)
        self.assertEqual(df1["marketing_area"]["2 Lapins"], 0)

        fruit_list_1 = []
        region_list_1 = []

        df2 = plantation.area_summary(fruit_list_1, self.region_list)
        self.assertTrue(df2.empty)

        df3 = plantation.area_summary(fruit_list_1, region_list_1)
        self.assertTrue(df3.empty)

        df4 = plantation.area_summary(self.fruit_list, region_list_1)
        self.assertEqual(df4["picking_area"]["1 Ambrosia"], 0)
        self.assertEqual(df4["picking_area"]["3 Elberta"], 0)
        self.assertEqual(df4["picking_area"]["2 Lapins"], 0)
        self.assertEqual(df4["marketing_area"]["1 Ambrosia"], 0)
        self.assertEqual(df4["marketing_area"]["3 Elberta"], 0)
        self.assertEqual(df4["marketing_area"]["2 Lapins"], 0)

    def test_region_loading(self):
        # test not exist csv
        d0 = plantation.region_loading("../production_file_test/test_not_exist.csv")
        self.assertTrue(d0.empty)

        # test empty csv
        d1 = plantation.region_loading("production_file_test/test_empty.csv")
        self.assertTrue(d1.empty)

        # test one line csv
        d2 = plantation.region_loading("production_file_test/test_one_region.csv")
        data = {
            'regionId': [1],
            'fruit_type_num': [1],
            'variety': ['Ambrosia'],
            'area': [20],
            'areaType': ['market']
        }
        df = pd.DataFrame(data)
        self.assertTrue(df.equals(d2))

        # test multiple line csv
        d3 = plantation.region_loading("production_file_test/test_multiple_region.csv")
        data2 = {
            'regionId': [1, 2, 3, 4],
            'fruit_type_num': [1, 3, 2, 2],
            'variety': ['Ambrosia', 'Elberta', 'Lapins', 'Lapins'],
            'area': [20, 30, 40, 40],
            'areaType': ['market', 'market', 'pick', 'pick']
        }
        df2 = pd.DataFrame(data2)
        self.assertTrue(df2.equals(d3))

    def test_fruit_index_list(self):
        i1, i2 = plantation.fruit_index_list(self.region_list, 1, "Ambrosia")
        self.assertEqual(i1, [])
        self.assertEqual(i2, [1])

        i3, i4 = plantation.fruit_index_list(self.region_list, 1, "Lapins")
        self.assertEqual(i3, [])
        self.assertEqual(i4, [])

        i5, i6 = plantation.fruit_index_list(self.region_list, 2, "Lapins")
        self.assertEqual(i5, [3, 4])
        self.assertEqual(i6, [])

        i7, i8 = plantation.fruit_index_list(self.region_list, 3, "Elberta")
        self.assertEqual(i7, [])
        self.assertEqual(i8, [2])

    def test_region_class_transfer(self):
        # test valid region dataframe
        data = {
            'regionId': [1],
            'fruit_type_num': [1],
            'variety': ['Ambrosia'],
            'area': [20],
            'areaType': ['market']
        }
        df = pd.DataFrame(data)
        r_list_1 = plantation.region_class_tranfer(df)
        self.assertEqual(len(r_list_1),1)
        self.assertTrue(isinstance(r_list_1[0], plantation.Region))

        # test empty region dataframe
        data2 = {
            'regionId': [],
            'fruit_type_num': [],
            'variety': [],
            'area': [],
            'areaType': []
        }
        df2 = pd.DataFrame(data2)
        r_list_2 = plantation.region_class_tranfer(df2)
        self.assertEqual([], r_list_2)

        # test region dataframe(more than one region)
        data3 = {
            'regionId': [1, 2, 3, 4],
            'fruit_type_num': [1, 3, 2, 2],
            'variety': ['Ambrosia', 'Elberta', 'Lapins', 'Lapins'],
            'area': [20, 30, 40, 40],
            'areaType': ['market', 'market', 'pick', 'pick']
        }
        df3 = pd.DataFrame(data3)
        r_list_3 = plantation.region_class_tranfer(df3)
        self.assertEqual(4,len(r_list_3))
        for i in range(0,len(r_list_3)):
            self.assertTrue(isinstance(r_list_3[i], plantation.Region))

        # test empty dataframe(no list/index)
        data4 = {}
        df4 = pd.DataFrame(data4)
        with self.assertRaises(Exception) as context:
            result = plantation.region_class_tranfer(df4)
        self.assertIsInstance(context.exception, AttributeError)

    def test_region_saving(self):
        file1 = "production_file_test/test_region.csv"
        plantation.region_saving(file1, self.region_list)

        with open(file1, 'r') as file:
            content = file.read()
        self.assertIn("regionId,fruit_type_num,variety,area,areaType", content)
        self.assertIn("1,1,Ambrosia,20,market", content)
        self.assertIn("2,3,Elberta,30,market", content)
        self.assertIn("3,2,Lapins,40,pick", content)
        self.assertIn("4,2,Lapins,40,pick", content)

        plantation.region_saving(file1, [])
        with open(file1, 'r') as file:
            content = file.read()
        self.assertIn("regionId,fruit_type_num,variety,area,areaType", content)
        self.assertNotIn("1,1,Ambrosia,20,market", content)

        plantation.region_saving(file1, [self.region_list[1]])
        with open(file1, 'r') as file:
            content = file.read()
        self.assertIn("regionId,fruit_type_num,variety,area,areaType", content)
        self.assertIn("2,3,Elberta,30,market", content)

        plantation.region_saving(file1, [self.r1, self.r3])
        with open(file1, 'r') as file:
            content = file.read()
        self.assertIn("regionId,fruit_type_num,variety,area,areaType", content)
        self.assertIn("1,1,Ambrosia,20,market", content)
        self.assertIn("3,2,Lapins,40,pick", content)

    def test_add_region(self):
        file1 = "production_file_test/test_region.csv"

        #check if add region with unexist fruit
        with self.assertRaises(plantation.FruitNoneException):
            plantation.add_region(2, 2, "Ambrosia", 20, "market", self.fruit_list, file1)

        # check if add region with existed regionId
        with self.assertRaises(plantation.RegionExistException):
            plantation.add_region(3, 1, "Ambrosia", 20, "market", self.fruit_list, file1)

        plantation.add_region(4, 1, "Ambrosia", 40, "pick",
                              self.fruit_list, file1)
        with open(file1, 'r') as file:
            content = file.read()
        self.assertIn("4,1,Ambrosia,40,pick", content)

        plantation.add_region(5, 2, "Lapins", 40, "pick",
                              self.fruit_list, file1)
        with open(file1, 'r') as file:
            content = file.read()
        self.assertIn("5,2,Lapins,40,pick", content)

if __name__ == '__main__':
    unittest.main()



import src.orchardmanagementpackage.production.fruit_info as fruit_info
import pandas as pd

"""
This module contains classes and functions related to fruit cultivation regions, including the Region class,
area calculations, region summary report, and data storage using DataFrames.
"""

class Region():
    """
    Represents a region with specific fruit cultivation information, including region ID, fruit type number,
    variety, area, and area type.

    Attributes:
    - regionId (int): The unique identifier for the region.
    - fruit_type_num (int): The type number of the fruit cultivated in the region.
    - variety (str): The variety of the fruit cultivated in the region.
    - area (float): The area of the region for fruit cultivation.
    - areaType (str): The type of the region, either "market" or "pick".
    """
    areaType_list = ["market", "pick"]

    def __init__(self, regionId, fruit_type_num, variety, area, areaType):
        """
        Initializes an instance of the Region class.
        """
        self.regionId = regionId
        self.fruit_type_num = fruit_type_num
        self.variety = variety
        self.area = area
        self.areaType = areaType

    # def __str__(self):
    #     """
    #     Returns a string representation of the region.
    #     """
    #     return f"{self.regionId}, {self.fruit_type_num}, {self.variety}, {self.area}, {self.areaType}"

    def get_area_type(self):
        """
        Returns the area type of the region.
        """
        return self.areaType

    def set_area_type(self, areaType):
        """
        Sets the area type of the region.
        """
        if areaType in Region.areaType_list:
            self.areaType = areaType
        else:
            print("Invalid area type")

    def get_area(self):
        """
        Returns the area of the region.
        """
        return self.area

    def get_fruit(self, fruit_list):
        """
        Retrieves the specific fruit instance for the region.
        """
        return fruit_info.get_fruit(self.fruit_type_num, self.variety, fruit_list)

    def region_display(self, fruit_list):
        """
        Displays information about the region and the cultivated fruit.
        """
        fruit = self.get_fruit(fruit_list)
        if isinstance(fruit, fruit_info.Apple):
            print(f"Region {self.regionId}: {self.variety} apple. Region Type: {self.areaType}. Area: {self.get_area()}")
        elif isinstance(fruit, fruit_info.Cherry):
            print(f"Region {self.regionId}: {self.variety} cherry. Region Type: {self.areaType}. Area: {self.get_area()}")
        else:
            print(f"Region {self.regionId}: {self.variety} peach. Region Type: {self.areaType}. Area: {self.get_area()}")


def get_region(region_index, region_list):
    """Retrieve a region from the list based on the region ID."""
    for region in region_list:
        if region.regionId == region_index:
            return region


def set_picking_region(plantingRegion):
    """Set the area type of the planting region to 'pick'."""
    plantingRegion.set_area_type("pick")
    return plantingRegion


def set_marketing_region(plantingRegion):
    """Set the area type of the planting region to 'market'."""
    plantingRegion.set_area_type("market")
    return plantingRegion


def area_amount_variety(plantation_list, fruit_type_num, fruit_variety):
    """Get the total planting area for a specific fruit type and variety."""
    area_pick_sum = 0
    area_market_sum = 0

    for region in plantation_list:
        f_type_num = region.fruit_type_num
        f_variety = region.variety

        if f_type_num == fruit_type_num and fruit_variety == f_variety:
            if region.get_area_type() == "market":
                area_market_sum += region.area
            else:
                area_pick_sum += region.area
    return area_pick_sum, area_market_sum


def fruit_index_list(plantation_list, fruit_type_num, fruit_variety):
    """Get lists of region IDs for a specific fruit type and variety based on area type."""
    fruit_index_pick = []
    fruit_index_market = []
    for region in plantation_list:
        if region.fruit_type_num == fruit_type_num and region.variety == fruit_variety:
            if region.get_area_type() == "market":
                fruit_index_market.append(region.regionId)
            else:
                fruit_index_pick.append(region.regionId)
    return fruit_index_pick, fruit_index_market

def region_summary(fruit_list, region_list):
    """
    Generate a summary DataFrame with picking and marketing area information for each fruit type and variety.

    Returns: DataFrame containing picking and marketing area information along with region indices.
    """
    area_dict_pick = {}
    area_dict_market = {}

    index_dict_pick = {}
    index_dict_market = {}

    for f in fruit_list:
        fruit_type_num = f.get_type_num()
        variety = f.variety
        display_name = variety+ " " + f.type

        picking_regions, marketing_regions = fruit_index_list(region_list, fruit_type_num, variety)
        picking_area, marketing_area = area_amount_variety(region_list, fruit_type_num, variety)

        area_dict_pick[display_name] = picking_area
        area_dict_market[display_name] = marketing_area

        # Fill lists to the same length
        max_len = max(len(picking_regions), len(marketing_regions))
        index_dict_pick[display_name] = picking_regions + [None] * (max_len - len(picking_regions))
        index_dict_market[display_name] = marketing_regions + [None] * (max_len - len(marketing_regions))

    index_dict_pick = {key: str(value) for key, value in index_dict_pick.items()}
    index_dict_market = {key: str(value) for key, value in index_dict_market.items()}

    # Create summary dataframes
    df_area_pick = pd.DataFrame(area_dict_pick, index=["pick_area"])
    #df_area_market = pd.DataFrame(area_dict_market, index=["market_area"])
    df_area_market = pd.DataFrame(area_dict_market, index=["market_area"])

    df_index_pick = pd.DataFrame(index_dict_pick, index = ["picking_regions_index"])
    df_index_market = pd.DataFrame(index_dict_market, index = ["marketing_regions_index"])

    df_combined = pd.concat([df_area_pick, df_index_pick, df_area_market, df_index_market], axis=0)
    return df_combined


def area_summary(fruit_list,region_list):
    """
    Generate a summary DataFrame with picking and marketing area information for each fruit type. This summary will be
    used for inventory management

    Returns: DataFrame containing picking and marketing area information.
    """
    df_combined = region_summary(fruit_list,region_list)
    df_area_combined = pd.concat([df_combined.loc["pick_area"], df_combined.loc["market_area"]], axis=1)
    df_area_combined = df_area_combined.rename(columns={"pick_area": "picking_area", "market_area": "marketing_area"})

    index_mapping = {"apple": "1", "cherry": "2", "peach": "3"}
    df_area_combined.index = df_area_combined.index.map(
        lambda x: f"{index_mapping.get(x.split()[1], x.split()[1])} {x.split()[0]}")

    return df_area_combined


def region_saving(plantation_file,region_list):
    """
    Save region information from a list of Region instances to a DataFrame and store it in a CSV file.
    CSV file columns: 'regionId', 'fruit_type_num', 'variety', 'area', 'areaType'.
    """
    data_region = region_loading(plantation_file)

    region_info_list = []
    for r in region_list:
        regionId = r.regionId
        fruit_type_num = r.fruit_type_num
        variety = r.variety
        area = r.area
        areaType = r.get_area_type()

        region_info = [regionId, fruit_type_num, variety, area, areaType]
        region_info_list.append(region_info)

    data_region = pd.DataFrame(region_info_list, columns=data_region.columns)
    data_region.to_csv(plantation_file, index=False, header=True)
    #print("successful save")


def region_loading(plantation_file):
    """
    Load region information from a CSV file and convert it into a DataFrame.
    """
    try:
        data_region = pd.read_csv(plantation_file)
    except (FileNotFoundError, pd.errors.EmptyDataError):
        title_list = ["regionId", "fruit_type_num", "variety", "area", "areaType"]
        data_region = pd.DataFrame(columns=title_list)
    return data_region


def region_class_tranfer(data_region):
    """
    Convert a DataFrame of region information into a list of Region class instances.
    """
    region_list = data_region.apply(
        lambda row: Region(row['regionId'], row['fruit_type_num'], row['variety'], row['area'],row['areaType']),
        axis=1).tolist()
    return region_list


class FruitNoneException(Exception):
    error_message = "planted fruit is not exist"
    def __init__(self, fruit):
        self.fruit = fruit

    def __str__(self):
        if self.fruit is None:
            return FruitNoneException.error_message

class RegionExistException(Exception):
    error_message = "input region is exist now"
    def __init__(self, regionId, regionId_set):
        self.regionId = regionId
        self.regionId_set = regionId_set

    def __str__(self):
        if self.regionId in self.regionId_set:
            return RegionExistException.error_message


def add_region(regionId, fruit_type_num, variety, area, areaType, fruit_list, planatation_file):
    """
    Add a new region to the list of regions.

    Note:
    add_region is used for plantation initialize, it will only call at the start of main.
    """
    data_region = region_loading(planatation_file)
    region_list = region_class_tranfer(data_region)
    regionId_set = set()
    for region in region_list:
        regionId_set.add(region.regionId)
    fruit = fruit_info.get_fruit(fruit_type_num, variety, fruit_list)
    if regionId in regionId_set:
        raise RegionExistException(regionId, regionId_set)
    elif fruit is None:
        raise FruitNoneException(fruit)
    else:
        region = Region(regionId, fruit_type_num, variety, area, areaType)
        region_list.append(region)
        region_saving(planatation_file, region_list)

    #
    # try:
    #     fruit = fruit_info.get_fruit(fruit_type_num, variety, fruit_list)
    #     if fruit is None:
    #         raise FruitNoneException(fruit)
    #     if regionId in regionId_set:
    #         raise RegionExistException(regionId, regionId_set)
    # except FruitNoneException as e:
    #     print(f"Fail to add region, {e}")
    # except RegionExistException as r:
    #     print(f"Fail to add region, {r}")
































import os
import os.path
import sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import unittest
import pandas as pd
from unittest.mock import patch
from io import StringIO
from src.orchardmanagementpackage.management.inventory import InventoryManagement
from src.orchardmanagementpackage.management.sales import SalesManagement


class TestSalesManagement(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("Setting up class...")
        # Initialize common data for the entire test class
        cls.inventory_origin_df = pd.read_csv("management_testing_files/inventory_origin.csv")
        cls.remaining_productivity_origin_df = pd.read_csv(
            "management_testing_files/remaining_productivity_origin.csv")
        cls.extra_productivity_origin_df = pd.read_csv("management_testing_files/extra_productivity_origin.csv")
        cls.orders_origin_df = pd.read_csv("management_testing_files/orders_origin.csv")

    @classmethod
    def tearDownClass(cls):
        print("Tearing down class...")
        # Clean up any resources for entire class
        cls.inventory_origin_df = None
        cls.remaining_productivity_origin_df = None
        cls.extra_productivity_origin_df = None
        cls.orders_origin_df = None

    def setUp(self):
        print("Setting up test...")
        # Create a temporary DataFrame for testing purposes
        self.inventory_origin_df.to_csv("inventory.csv", index=False)
        self.remaining_productivity_origin_df.to_csv("remaining_productivity.csv", index=False)
        self.extra_productivity_origin_df.to_csv("extra_productivity.csv", index=False)
        self.orders_origin_df.to_csv("orders.csv", index=False)
        self.inventory_manager = InventoryManagement("extra_productivity.csv",
                                                     "remaining_productivity.csv", "inventory.csv")
        self.sales_manager = SalesManagement(self.inventory_manager, "orders.csv")

    def tearDown(self):
        print("Tearing down test...")
        # Clean up any resources used during the test
        del self.inventory_manager
        files_to_remove = ["inventory.csv", "orders.csv", "remaining_productivity.csv", "extra_productivity.csv"]
        for file_name in files_to_remove:
            if os.path.exists(file_name):
                os.remove(file_name)

    @patch('sys.stdout', new_callable=StringIO)
    def test_add_order(self, mock_stdout):
        # wrong sales_type
        self.assertEqual(self.sales_manager.add_order('sale', 'Ambrosia', 10),False, "add_order abnormal")

        # Fruit type 1
        self.sales_manager.add_order('selling', 'Ambrosia', 50)
        self.assertListEqual(list(self.sales_manager.orders.iloc[3]),
                             [4, "selling", 1, "Ambrosia", 50, 1.2, 60.0], "add_order abnormal")
        # Fruit type 2
        self.sales_manager.add_order('picking', 'Lapins', 10)
        self.assertListEqual(list(self.sales_manager.orders.iloc[4]),
                             [5, "picking", 2, "Lapins", 10, 2.99, 29.9], "add_order abnormal")
        # Fruit type 3
        self.sales_manager.add_order('selling', 'Elberta', 20)
        self.assertListEqual(list(self.sales_manager.orders.iloc[5]),
                             [6, 'selling', 3, 'Elberta', 20, 1.2, 24.0], "add_order abnormal")
        # wrong variety
        self.sales_manager.add_order('selling', 'Amb', 10)
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Fruit 'Amb' not found.", printed_output, "add_order abnormal")

        # Variety with no instance
        with self.assertRaises(ValueError) as context:
            self.sales_manager.add_order("selling", "Cresthaven", 10)
        self.assertEqual(str(context.exception), "There is no instance of this fruit",
                         "add_order abnormal")

        # Empty orders table
        self.sales_manager.orders = pd.read_csv("orders.csv", nrows=0)
        self.sales_manager.orders.to_csv("orders.csv", index=False)
        self.sales_manager.add_order('selling', 'Ambrosia', 10)
        self.assertListEqual(list(self.sales_manager.orders.iloc[0]),
                             [1, "selling", 1, "Ambrosia", 10, 1.2, 12], "add_order abnormal")

    def test_order_display(self):
        self.assertEqual(self.sales_manager.order_display(), 3, "order_display abnormal")

    @patch('sys.stdout', new_callable=StringIO)
    def test_revenue_display(self, mock_stdout):
        # variable not found
        self.sales_manager.revenue_display("random")
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Variable random not found. Please choose from: sales_type or fruit_type.", printed_output,
                      "revenue_display abnormal when variable not found")
        # display by sales_type
        result = self.sales_manager.revenue_display("sales_type")
        expected_result = pd.DataFrame({
            'sales_type': ['picking', 'selling'],
            'revenue': [119.6, 120.0]
        })
        self.assertTrue(result.equals(expected_result), "Actual result does not match expected result")
        # display by fruit_type
        result2 = self.sales_manager.revenue_display('fruit_type')
        expected_result2 = pd.DataFrame({
            'fruit_type': [1, 2, 3],
            'revenue': [60.0, 119.6, 60.0]
        })
        self.assertTrue(result2.equals(expected_result2), "Actual result does not match expected result")
        # Empty orders table
        self.sales_manager.orders = pd.read_csv("orders.csv", nrows=0)
        self.sales_manager.orders.to_csv("orders.csv", index=False)
        self.sales_manager.revenue_display('fruit_type')
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("No data available.",
                      printed_output, "revenue_display abnormal when orders empty")

    @patch('sys.stdout', new_callable=StringIO)
    def test_check_inventory(self, mock_stdout):
        self.sales_manager.check_inventory("Ambrosia", 10)
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Inventory of Ambrosia is enough for this order.",
                      printed_output, "check_inventory abnormal")
        self.sales_manager.check_inventory("Ambrosia", 1000)
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Remaining productivity of Ambrosia is 300 kg, which is not enough",
                      printed_output, "check_inventory abnormal")
        self.sales_manager.check_inventory("Amb", 1000)
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Error: Amb is not a valid fruit type.",
                      printed_output, "check_inventory abnormal")



if __name__ == '__main__':
    unittest.main()



from src.orchardmanagementpackage import management as inventory


def inventory_operator(extra_productivity_file, remaining_productivity_file, inventory_file):
    # Instantiate the InventoryManagement class
    inventory_manager = inventory.InventoryManagement(extra_productivity_file, remaining_productivity_file, inventory_file)
    plotter = inventory.HistoricalPlotter(remaining_productivity_file, inventory_file)

    while True:
        print("Inventory Operations:")
        print("1. View Current Inventory")
        print("2. View Remaining Productivity")
        print("3. Add Inventory")
        print("4. Remove Inventory")
        print("5. View production more than estimated")
        print("6. View the plot of inventory history data ")
        print("0. Back to Main Menu")

        operation_choice = input("Please enter operation index here: ")

        try:
            operation_choice = int(operation_choice)
            if operation_choice == 1:
                inventory_manager.get_current_inventory()
            elif operation_choice == 2:
                inventory_manager.get_remaining_productivity()
            elif operation_choice == 3:
                fruit_type = input(
                    "Enter fruit type(choose from Ambrosia,Gala,Honeycrisp,Lapins,Sweetheart,Skeena,Redhaven,Elberta,Cresthaven): ")
                quantity = int(input("Enter quantity to add: "))
                inventory_manager.add_inventory(fruit_type, quantity)
            elif operation_choice == 4:
                fruit_type = input("Enter fruit type(choose from Ambrosia,Gala,Honeycrisp,Lapins,Sweetheart,Skeena,Redhaven,Elberta,Cresthaven):")
                quantity = int(input("Enter quantity to remove: "))
                inventory_manager.remove_inventory(fruit_type, quantity)
            elif operation_choice == 5:
                inventory_manager.get_extra_productivity()
            elif operation_choice == 6:
                fruit_type = int(input("Enter fruit type(1-Apple, 2-Cherry, 3-Peach):"))
                file_type = int(input("Enter 1 for inventory and 2 for remaining capacity:"))
                plotter.plot_historical_data(fruit_type, file_type)
            elif operation_choice == 0:
                print("Back to the Main Menu.")
                break
            else:
                print("Invalid operation index.")
        except ValueError:
            print("Invalid input, please enter operation index.")



import os.path
import sys
import unittest

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

import src.orchardmanagementpackage.production.fruit_info as fruit_info


class FruitTestCase(unittest.TestCase):

    def setUp(self):
        self.a1 = fruit_info.Apple("Ambrosia", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
        self.c1 = fruit_info.Cherry("Lapins", "small", "median sweet", "median sour", "soft", 2.99, "cans")
        self.p1 = fruit_info.Peach("Elberta", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
        self.p2 = fruit_info.Peach("Redhaven", "median", "median sweet", "less sour", "crunchy", 3.2, "pie, can")
        self.fruit_list = [self.a1,self.p1,self.c1, self.p2]

    def tearDown(self):
        del self.a1
        del self.c1
        del self.p1
        del self.p2
        del self.fruit_list

    def test_describe(self):
        self.a1_describe = "big, less sour, very sweet, crunchy.It is good for pie"
        self.c1_describe = "small, median sour, median sweet, soft.It is good for cans"
        self.p1_describe = "big, less sour, very sweet, crunchy.It is good for pie"
        self.p2_describe = "median, less sour, median sweet, crunchy.It is good for pie, can"

        self.assertEqual(self.a1.describe(), self.a1_describe)
        self.assertEqual(self.c1.describe(), self.c1_describe)
        self.assertEqual(self.p1.describe(), self.p1_describe)
        self.assertEqual(self.p2.describe(), self.p2_describe)

    def test_price_get(self):
        self.assertEqual(self.a1.get_price(), 1.2)
        self.assertEqual(self.c1.get_price(), 2.99)
        self.assertEqual(self.p1.get_price(), 1.2)
        self.assertEqual(self.p2.get_price(), 3.2)

    def test_price_set(self):
        self.a1.set_price(1.9)
        self.c1.set_price(1.6)
        self.p1.set_price(2.1)
        self.p2.set_price(3.5)

        self.assertEqual(self.a1.get_price(), 1.9)
        self.assertEqual(self.c1.get_price(), 1.6)
        self.assertEqual(self.p1.get_price(), 2.1)
        self.assertEqual(self.p2.get_price(), 3.5)

    def test_get_type_num(self):
        self.assertEqual(self.a1.get_type_num(), 1)
        self.assertEqual(self.c1.get_type_num(), 2)
        self.assertEqual(self.p1.get_type_num(), 3)
        self.assertEqual(self.p1.get_type_num(), 3)

    def test_get_available_season(self):
        self.assertTrue(self.a1.get_available_season(9))
        self.assertFalse(self.c1.get_available_season(9))
        self.assertTrue(self.p1.get_available_season(9))
        self.assertFalse(self.p2.get_available_season(9))


class FruitInfoTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.a1 = fruit_info.Apple("Ambrosia", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
        cls.c1 = fruit_info.Cherry("Lapins", "small", "median sweet", "median sour", "soft", 2.99, "cans")
        cls.p1 = fruit_info.Peach("Elberta", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
        cls.p2 = fruit_info.Peach("Redhaven", "median", "median sweet", "less sour", "crunchy", 3.2, "pie, can")
        cls.fruit_list = [cls.a1,cls.p1,cls.c1,cls.p2]

    @classmethod
    def tearDownClass(cls):
        del cls.a1
        del cls.c1
        del cls.p1
        del cls.p2
        del cls.fruit_list
        print("Finish FruitInfoTestCase")

    def test_datetime_transfer(self):
        self.assertEqual(fruit_info.datetime_transfer("09-21"), 9)
        self.assertEqual(fruit_info.datetime_transfer("09-04"), 9)
        self.assertEqual(fruit_info.datetime_transfer("12-21"), 12)
        self.assertEqual(fruit_info.datetime_transfer("9-21"), 9)

    def test_fruit_available_check(self):
        self.assertTrue(fruit_info.fruit_available_check("09-21", self.a1))
        self.assertFalse(fruit_info.fruit_available_check("09-21", self.c1))
        self.assertFalse(fruit_info.fruit_available_check("12-21", self.p1))
        self.assertTrue(fruit_info.fruit_available_check("08-21", self.p2))


    def test_get_fruit(self):
        self.assertEqual(fruit_info.get_fruit(1, "Ambrosia", self.fruit_list), self.a1)
        self.assertEqual(fruit_info.get_fruit(2, "Lapins", self.fruit_list), self.c1)
        self.assertEqual(fruit_info.get_fruit(3, "Elberta", self.fruit_list), self.p1)
        self.assertEqual(fruit_info.get_fruit(3, "Redhaven", self.fruit_list), self.p2)

    def test_file_load(self):
        # test non exist file
        self.assertIsNone(fruit_info.file_load("../production_file_test/test_non_exist.csv"))
        # test empty file
        self.assertEqual(fruit_info.file_load("production_file_test/test_empty.csv"), [])
        # test file with one line file
        self.assertEqual(fruit_info.file_load("production_file_test/test_one.csv"),
                         [['1', 'Ambrosia', 'big', 'very sweet', 'less sour', 'crunchy', '1.2', 'pie']])
        # test file with multiple line
        self.assertEqual(fruit_info.file_load("production_file_test/test_mutiple.csv"),
                         [['1', 'Ambrosia', 'big', 'very sweet', 'less sour', 'crunchy', '1.2', 'pie'],
                          ['3', 'Elberta', 'big', 'very sweet', 'less sour', 'crunchy', '1.2', 'pie'],
                          ['2','Lapins','small','median sweet','median sour','soft','2.99', 'cans']])

    def test_fruit_class_load(self):
        # test non-exist file
        self.assertIsNone(fruit_info.fruit_class_load("../production_file_test/test_non_exist.csv"))
        # test empty file
        self.assertEqual(fruit_info.fruit_class_load("production_file_test/test_empty.csv"), [])
        # test one-line file
        load_a1 = fruit_info.fruit_class_load("production_file_test/test_one.csv")
        self.assertEqual(len(load_a1),1)
        self.assertEqual(load_a1[0].get_type_num(), self.a1.get_type_num())
        self.assertEqual(load_a1[0].describe(),self.a1.describe())
        # test multiple-line file
        load_fruit_list = fruit_info.fruit_class_load("production_file_test/test_mutiple.csv")
        self.assertEqual(len(load_fruit_list),3)
        for i in range(0,len(load_fruit_list)):
            self.assertEqual(load_fruit_list[i].get_type_num(), self.fruit_list[i].get_type_num())
            self.assertEqual(load_fruit_list[i].describe(), self.fruit_list[i].describe())

    def test_file_store(self):
        fruit_info.file_store("production_file_test/test_save.csv", [[1, 2, 3]])
        with open("production_file_test/test_save.csv", 'r') as file:
            content = file.read()
        self.assertIn("1,2,3",content)

        fruit_info.file_store("production_file_test/test_save.csv", [[1, 2, 3], [4, 5, 6]])
        with open("production_file_test/test_save.csv", 'r') as file:
            content = file.read()
        self.assertIn("1,2,3", content)
        self.assertIn("4,5,6",content)

        fruit_info.file_store("production_file_test/test_save.csv", [["abc", "bcd", "ace"]])
        with open("production_file_test/test_save.csv", 'r') as file:
            content = file.read()
        self.assertIn("abc,bcd,ace", content)

        fruit_info.file_store("production_file_test/test_save.csv", [["abc", 1, "bcd", 2, "ace"], ["efg", 1, "bcf", 2, "ace"]])
        with open("production_file_test/test_save.csv", 'r') as file:
            content = file.read()
        self.assertIn("abc,1,bcd,2,ace", content)
        self.assertIn("efg,1,bcf,2,ace", content)

    def test_fruit_information_store(self):
        csv_test_save = "production_file_test/test_save.csv"
        fruit_list_1 = [self.a1]
        fruit_list_2 = [self.a1, self.c1]
        fruit_list_3 = [self.p1, self.p2]
        fruit_list_4 = [self.a1, self.p1]

        fruit_info.fruit_information_store(fruit_list_1, csv_test_save)
        with open(csv_test_save, 'r') as file:
            content = file.read()
        self.assertIn("1,Ambrosia,big,very sweet,less sour,crunchy,1.2,pie", content)

        fruit_info.fruit_information_store(fruit_list_2, csv_test_save)
        with open(csv_test_save, 'r') as file:
            content = file.read()
        self.assertIn("1,Ambrosia,big,very sweet,less sour,crunchy,1.2,pie", content)
        self.assertIn("2,Lapins,small,median sweet,median sour,soft,2.99,cans", content)

        fruit_info.fruit_information_store(fruit_list_3, csv_test_save)
        with open(csv_test_save, 'r') as file:
            content = file.read()
        self.assertNotIn("1,Ambrosia,big,very sweet,less sour,crunchy,1.2,pie", content)
        self.assertIn("3,Elberta,big,very sweet,less sour,crunchy,1.2,pie", content)
        self.assertIn("3,Redhaven,median,median sweet,less sour,crunchy,3.2,\"pie, can\"", content)

        fruit_info.fruit_information_store(fruit_list_4, csv_test_save)
        with open(csv_test_save, 'r') as file:
            content = file.read()
        self.assertIn("1,Ambrosia,big,very sweet,less sour,crunchy,1.2,pie", content)
        self.assertIn("3,Elberta,big,very sweet,less sour,crunchy,1.2,pie", content)

class FruitInfoAddRemoveTestCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.a1 = fruit_info.Apple("Ambrosia", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
        cls.c1 = fruit_info.Cherry("Lapins", "small", "median sweet", "median sour", "soft", 2.99, "cans")
        cls.p1 = fruit_info.Peach("Elberta", "big", "very sweet", "less sour", "crunchy", 1.2, "pie")
        cls.p2 = fruit_info.Peach("Redhaven", "median", "median sweet", "less sour", "crunchy", 3.2, "pie, can")
        cls.fruit_list = [cls.a1, cls.p1, cls.c1, cls.p2]
        cls.file_path = "production_file_test/test_fruit.csv"

        with open(cls.file_path, 'w', newline='') as csvfile:
            csvfile.truncate(0)

    @classmethod
    def tearDownClass(cls):
        del cls.a1
        del cls.c1
        del cls.p1
        del cls.p2
        del cls.fruit_list
        del cls.file_path
        print("Finish FruitInfoAddRemoveTestCase")

    def test_add_fruit(self):
        fruit_info.add_fruit(1, "Ambrosia", "big", "very sweet", "less sour", "crunchy", 1.2, "pie", self.file_path)
        load_fruit_list = fruit_info.fruit_class_load(self.file_path)
        self.assertEqual(load_fruit_list[0].get_type_num(), self.a1.get_type_num())
        self.assertEqual(load_fruit_list[0].describe(), self.a1.describe())

        fruit_info.add_fruit(1, "Ambrosia", "small", "very sweet", "less sour", "crunchy", 1.2, "pie", self.file_path)
        with open(self.file_path, 'r') as file:
            content = file.read()
        self.assertNotIn("1,Ambrosia,small,very sweet,less sour,crunchy,1.2,pie", content)

        fruit_info.add_fruit(3, "Elberta", "big", "very sweet", "less sour", "crunchy", 1.2, "pie", self.file_path)
        load_fruit_list = fruit_info.fruit_class_load(self.file_path)
        for i in range(0,len(load_fruit_list)):
            self.assertEqual(load_fruit_list[i].get_type_num(), self.fruit_list[i].get_type_num())
            self.assertEqual(load_fruit_list[i].describe(), self.fruit_list[i].describe())

        self.assertFalse(fruit_info.add_fruit(2, "abc", "small", "median sweet", "median sour", "soft", 2.99, "cans", self.file_path))
        load_fruit_list = fruit_info.fruit_class_load(self.file_path)
        for i in range(0, len(load_fruit_list)):
            self.assertEqual(load_fruit_list[i].get_type_num(), self.fruit_list[i].get_type_num())
            self.assertEqual(load_fruit_list[i].describe(), self.fruit_list[i].describe())


    def test_remove(self):
        with open(self.file_path, 'w', newline='') as csvfile:
            csvfile.truncate(0)

        fruit_info.add_fruit(1, "Ambrosia", "small", "very sweet", "less sour", "crunchy", 1.2, "pie", self.file_path)
        # test successfully remove
        fruit_info.remove_fruit(1, "Ambrosia", self.file_path)
        with open(self.file_path, 'r') as file:
            content = file.read()
        self.assertNotIn("1,Ambrosia,small,very sweet,less sour,crunchy,1.2,pie", content)

        fruit_info.add_fruit(1, "Ambrosia", "small", "very sweet", "less sour", "crunchy", 1.2, "pie", self.file_path)
        # test not able to remove with unpaired type_num and variety
        fruit_info.remove_fruit(2, "Ambrosia", self.file_path)
        with open(self.file_path, 'r') as file:
            content = file.read()
        self.assertIn("1,Ambrosia,small,very sweet,less sour,crunchy,1.2,pie", content)

        fruit_info.add_fruit(2, "Lapins", "small", "median sweet", "median sour", "soft", 2.99, "cans", self.file_path)
        # test not able to remove with wrong type_num and varitey
        fruit_info.remove_fruit(3, "ABC", self.file_path)
        with open(self.file_path, 'r') as file:
            content = file.read()
        self.assertIn("1,Ambrosia,small,very sweet,less sour,crunchy,1.2,pie", content)
        self.assertIn("2,Lapins,small,median sweet,median sour,soft,2.99,cans",content)

        fruit_info.remove_fruit(4, "Lapins", self.file_path)
        with open(self.file_path, 'r') as file:
            content = file.read()
        self.assertIn("1,Ambrosia,small,very sweet,less sour,crunchy,1.2,pie", content)
        self.assertIn("2,Lapins,small,median sweet,median sour,soft,2.99,cans", content)


if __name__ == '__main__':
    unittest.main()



import src.orchardmanagementpackage.production.fruit_info as fruit_info
import src.orchardmanagementpackage.production.plantation as plantation


def region_operations(fruit_file, plantation_file):
    """
    Manage orchard region operations.

    Usage:
    region_operations(fruit_file, plantation_file)
    - 'fruit_file': Path to the file containing fruit information.
    - 'plantation_file': Path to the file containing orchard region information.
    """

    data_region = plantation.region_loading(plantation_file)
    region_list = plantation.region_class_tranfer(data_region)
    fruits_list = fruit_info.fruit_class_load(fruit_file)

    while True:
        print("1. View orchard region report")
        print("2. Switch region type")
        print("0. Back to main menu")

        region_choice_input = input("Please enter your operation index: ")
        try:
            region_choice_input = int(region_choice_input)
            if region_choice_input == 1:
                for region in region_list:
                    region.region_display(fruits_list)
                df_summary = plantation.region_summary(fruits_list, region_list)
                print(df_summary)
            elif region_choice_input == 2:
                region_switch(plantation_file,region_list)
            elif region_choice_input == 0:
                print("Welcome to the Orchard Management Software")
                break
            else:
                print("Invalid operation index")

        except ValueError:
            print("Invalid operation index")


def region_switch(plantation_file, region_list):
    """
    Switch the area_type of an orchard region.
    """
    region_available = []
    for region in region_list:
        region_available.append(region.regionId)
    print(f"Available regions: {region_available}")
    regionId_input = input("Please enter regionId: ")
    try:
        regionId_input = int(regionId_input)
        if regionId_input in region_available:
            region_change = plantation.get_region(regionId_input, region_list)

            type_input = input("Please enter the new region area type, pick or market: ")
            if type_input == "pick":
                plantation.set_picking_region(region_change)
            elif type_input == "market":
                plantation.set_marketing_region(region_change)
            else:
                print("Invalid region area type")
            #save changes
            plantation.region_saving(plantation_file, region_list)
            print("successful save")
        else:
            print("Invalid regionId")
    except ValueError:
        print("Invalid input, please enter a valid regionId.")


from src.orchardmanagementpackage.management.inventory import InventoryManagement
from src.orchardmanagementpackage.management.sales import SalesManagement


def sales_operator(extra_productivity_file, remaining_productivity_file, inventory_file, order_file):
    # Instantiate the InventoryManagement class
    inventory_manager = InventoryManagement(extra_productivity_file, remaining_productivity_file, inventory_file)

    # Instantiate the SalesManagement class with the inventory manager
    sales_manager = SalesManagement(inventory_manager, order_file)

    while True:
        print("Sales Operations:")
        print("1. Add Sales Order")
        print("2. Add Picking Order")
        print("3. Display Orders")
        print("4. Display Revenue by Sales Type or Fruit Type")
        print("5. Plot Revenue Distribution by Fruit Type")
        print("0. Back to Main Menu")

        operation_choice = input("Please enter operation index here: ")

        try:
            operation_choice = int(operation_choice)
            if operation_choice == 1:
                fruit_variety = input("Enter fruit variety(choose from Ambrosia, Lapins, Elberta): ")
                weight = float(input("Enter weight: "))
                sales_manager.check_inventory(fruit_variety, weight)
                sales_manager.add_order("selling", fruit_variety, weight)
            elif operation_choice == 2:
                fruit_variety = input("Enter fruit variety(choose from Ambrosia, Lapins, Elberta): ")
                weight = float(input("Enter weight: "))
                sales_manager.add_order("picking", fruit_variety, weight)
                print("saving successfully")
            elif operation_choice == 3:
                sales_manager.order_display()
            elif operation_choice == 4:
                variable = input("Enter variable (sales_type or fruit_type): ")
                sales_manager.revenue_display(variable)
            elif operation_choice == 5:
                sales_manager.revenue_plotter()
            elif operation_choice == 0:
                print("Back to the Main Menu.")
                break
            else:
                print("Invalid operation index.")
        except ValueError:
            print("Invalid input, please enter operation index.")



import pandas as pd
import matplotlib.pyplot as plt


class SalesManagement:
    """
    A class for managing sales operations, including order processing, revenue analysis, and inventory checks.

    Attributes:
    - inventory_manager (InventoryManagement): An instance of the InventoryManagement class.
    - fruit_file (str): The path to the file containing fruit information.
    - order_file (str): The path to the file containing sales orders.
    - orders (pd.DataFrame): DataFrame to store sales order data.
    """

    def __init__(self, inventory_manager, order_file):
        """
        Initialize a SalesManagement instance.
        """
        self.inventory_manager = inventory_manager
        self.order_file = order_file
        self.orders = self.load_data()

    def load_data(self):
        """
        Load sales order data from a CSV file.

        Returns:
        - pd.DataFrame: DataFrame containing sales order data.
        """
        columns = ["index", "sales_type", "fruit_type", "fruit_variety", "weight", "unit_price", "revenue"]
        try:
            orders_data = pd.read_csv(self.order_file)
        except FileNotFoundError:
            # If the file doesn't exist, return an initial empty DataFrame
            orders_data = pd.DataFrame(columns=columns)
            orders_data.to_csv(self.order_file, index=True)
        return orders_data

    def add_order(self, sales_type, fruit_variety, weight):
        """
        Add a new sales order.

        Parameters:
        - sales_type (str): The type of sales operation ('selling' or 'picking').
        - fruit_variety (str): The variety of fruit for the order.
        - weight (float): The weight of the fruit in the order.
        """
        if fruit_variety not in self.inventory_manager.inventory.columns:
            print(f"Fruit '{fruit_variety}' not found.")
            return
        else:
            # file setting
            if sales_type == "selling":
                if not self.inventory_manager.remove_inventory(fruit_variety, weight):
                    print("Order cannot be added.")
                else:
                    self.inventory_manager.remove_inventory(fruit_variety, weight)
                    print("Order added")
            elif sales_type == "picking":
                self.inventory_manager.remove_remaining_productivity(fruit_variety, weight)
                print("Order added")
            else:
                print("sales_type not found, please choose from selling and picking")
                return False
            # index setting
            if self.orders.empty:
                new_index = 1
            else:
                new_index = self.orders["index"].max() + 1
            # fruit variety setting
            if fruit_variety in ["Ambrosia", "Gala", "Honeycrisp"]:
                fruit_type_num = 1
            elif fruit_variety in ["Lapins", "Sweetheart", "Skeena"]:
                fruit_type_num = 2
            elif fruit_variety in ["Redhaven", "Elberta", "Cresthaven"]:
                fruit_type_num = 3
            else:
                print("Variety not found")
                return False

            if fruit_variety == "Ambrosia":
                unit_price = 1.20
            elif fruit_variety == "Elberta":
                unit_price = 1.20
            elif fruit_variety == "Lapins":
                unit_price = 2.99
            else:
                raise ValueError("There is no instance of this fruit")

            new_order = pd.DataFrame(columns=self.orders.columns)
            new_order = pd.DataFrame({
                "index": [new_index],
                "sales_type": [sales_type],
                "fruit_type": [fruit_type_num],
                "fruit_variety": [fruit_variety],
                "weight": [weight],
                "unit_price": [unit_price],
                "revenue": [round(weight * unit_price, 2)]
            })

            self.orders = pd.concat([self.orders, new_order], ignore_index=True)
            self.orders.to_csv(self.order_file, index=False)
            return True

    def order_display(self):
        """
        Display the sales order data.
        """
        print(self.orders)
        return len(self.orders)

    def revenue_display(self, variable):
        """
        Display revenue data based on a specified variable.

        Parameters:
        - variable (str): The variable for grouping revenue data.
        """
        if variable not in self.orders.columns:
            print(f"Variable {variable} not found. Please choose from: sales_type or fruit_type.")
            return

        if not self.orders.empty:
            # Display the entire DataFrame
            revenue_by_variable = self.orders.groupby(variable)['revenue'].sum().reset_index()
            print(revenue_by_variable)
            return pd.DataFrame(revenue_by_variable)
        else:
            print("No data available.")

    def revenue_plotter(self):
        """
        Plot a pie chart of revenue distribution by fruit type.
        """
        if self.orders.empty:
            print("No data available for plotting.")
            return

        fruit_type_mapping = {
            1: 'Apple',
            2: 'Cherry',
            3: 'Peach'
        }

        self.orders['fruit_type_label'] = self.orders['fruit_type'].map(fruit_type_mapping)
        revenue_by_fruit_type = self.orders.groupby('fruit_type_label')['revenue'].sum().reset_index()

        plt.figure(figsize=(8, 8))
        plt.pie(revenue_by_fruit_type['revenue'], labels=revenue_by_fruit_type['fruit_type_label'],
                autopct='%1.1f%%', startangle=140)
        plt.title('Revenue Distribution by Fruit Type')
        plt.show()

    def check_inventory(self, fruit_variety, weight):
        """
        Checks if selling a specified quantity of fruits is possible.

        Parameters:
        - fruit_variety (str): The variety of fruit for the inventory check.
        - weight (float): The weight of the fruit to be sold.
        """
        current_inventory = self.inventory_manager.inventory.iloc[[-1]].copy()

        if fruit_variety in current_inventory.columns:
            inventory_weight = current_inventory[fruit_variety].iloc[0]
            print(inventory_weight)
            if inventory_weight >= weight:
                print(f"Inventory of {fruit_variety} is enough for this order.")
            else:
                print(f"Remaining productivity of {fruit_variety} is {inventory_weight} kg, which is not enough")
        else:
            print(f"Error: {fruit_variety} is not a valid fruit type.")



import os
import os.path
import sys
import unittest

sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
import pandas as pd
from unittest.mock import patch
from io import StringIO
from src.orchardmanagementpackage.management.inventory import InventoryManagement, HistoricalPlotter


class TestInventoryManagement(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("Setting up class...")
        # Initialize common data for the entire test class
        cls.inventory_origin_df = pd.read_csv("management_testing_files/inventory_origin.csv")
        cls.remaining_productivity_origin_df = pd.read_csv(
            "management_testing_files/remaining_productivity_origin.csv")
        cls.extra_productivity_origin_df = pd.read_csv(
            "management_testing_files/extra_productivity_origin.csv")

    @classmethod
    def tearDownClass(cls):
        print("Tearing down class...")
        # Clean up any resources for entire class
        cls.inventory_origin_df = None
        cls.remaining_productivity_origin_df = None
        cls.extra_productivity_origin_df = None

    def setUp(self):
        print("Setting up test...")
        # Create a temporary DataFrame for testing purposes
        self.inventory_origin_df.to_csv("inventory.csv", index=False)
        self.remaining_productivity_origin_df.to_csv("remaining_productivity.csv", index=False)
        self.extra_productivity_origin_df.to_csv("extra_productivity.csv", index=False)
        self.inventory_manager = InventoryManagement("extra_productivity.csv",
                                                     "remaining_productivity.csv", "inventory.csv")

    def tearDown(self):
        print("Tearing down test...")
        # Clean up any resources used during the test
        del self.inventory_manager
        files_to_remove = ["inventory.csv", "orders.csv", "remaining_productivity.csv", "extra_productivity.csv"]
        for file_name in files_to_remove:
            if os.path.exists(file_name):
                os.remove(file_name)


    def test_total_product_estimate(self):
        self.inventory_manager.remaining_productivity = pd.read_csv("remaining_productivity.csv", nrows=0)
        self.inventory_manager.remaining_productivity.to_csv("remaining_productivity.csv", index=False)
        self.inventory_manager.total_product_estimate()
        # normal condition
        self.assertListEqual(list(self.inventory_manager.remaining_productivity.iloc[0]),
                             [1, 30000, 0, 0, 40000, 0, 0, 0, 72000, 0], "First rows are not equal")
        with self.assertRaises(ValueError) as context:
            self.inventory_manager.total_product_estimate()
        # As this is the initiation of remaining_productivity raise error when file is not empty
        self.assertEqual(str(context.exception), "File is not empty, and cannot be initialized by total capacity")

    def test_add_inventory(self):
        self.inventory_manager.add_inventory("Ambrosia", 10)
        self.assertListEqual(list(self.inventory_manager.inventory.iloc[2]),
                             [3, 310, 0, 0, 0, 0, 0, 0, 400, 0], "add_inventory abnormal")
        with self.assertRaises(ValueError) as context:
            self.inventory_manager.add_inventory("Ambro", 10)
        # Fruit not found
        self.assertEqual(str(context.exception), "Fruit not found in inventory")

    def test_add_extra_productivity(self):
        self.inventory_manager.add_inventory("Gala", 100)  # Gala's remaining capacity is 0 and will add on extra file
        self.assertListEqual(list(self.inventory_manager.extra_productivity.iloc[1]),
                             [2, 0, 200, 50, 0, 0, 0, 0, 0, 0], "add_extra_productivity abnormal")

    @patch('sys.stdout', new_callable=StringIO)
    def test_remove_inventory(self, mock_stdout):
        # normal condition
        self.inventory_manager.remove_inventory("Ambrosia", 100)
        self.assertListEqual(list(self.inventory_manager.inventory.iloc[2]),
                             [3, 200, 0, 0, 0, 0, 0, 0, 400, 0], "remove_inventory abnormal")
        self.inventory_manager.remove_inventory("Elberta", 100)
        self.assertListEqual(list(self.inventory_manager.inventory.iloc[3]),
                             [4, 200, 0, 0, 0, 0, 0, 0, 300, 0], "remove_inventory abnormal")

        # fruit not exists
        with self.assertRaises(ValueError) as context:
            self.inventory_manager.add_inventory("Ambro", 10)
        self.assertEqual(str(context.exception), "Fruit not found in inventory",
                         "remove_inventory abnormal when fruit not found")

        # inventory not enough
        self.inventory_manager.remove_inventory("Ambrosia", 500)
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Not enough Ambrosia in inventory, only 200 left", printed_output,
                      "remove_inventory abnormal when lack of inventory")

        # inventory empty
        self.inventory_manager.inventory = pd.read_csv("inventory.csv", nrows=0)
        self.inventory_manager.inventory.to_csv("inventory.csv", index=False)

        self.inventory_manager.remove_inventory("Elberta", 100)
        printed_output2 = mock_stdout.getvalue().strip()
        self.assertIn("Inventory is empty.", printed_output2, "remove_inventory abnormal when inventory is empty")

    @patch('sys.stdout', new_callable=StringIO)
    def test_remove_remaining_productivity(self, mock_stdout):
        # normal condition
        self.inventory_manager.remove_remaining_productivity("Ambrosia", 500)
        self.assertListEqual(list(self.inventory_manager.remaining_productivity.iloc[-1]),
                             [2, 29500, 0, 0, 40000, 0, 0, 0, 72000, 0], "remove_remaining_productivity abnormal")

        # Fruit not exists
        self.inventory_manager.remove_remaining_productivity("Elb", 100)
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Fruit 'Elb' not found", printed_output,
                      "remove_remaining_productivity abnormal when fruit not found")

        # the inventory added exceed the remaining capacity
        self.inventory_manager.add_inventory("Gala", 100)
        printed_output2 = mock_stdout.getvalue().strip()
        self.assertIn("The product of Gala exceeds expectations and will be recorded in the extra_productivity file.",
                      printed_output2,
                      "remove_remaining_productivity abnormal when inventory exceed estimate")

        # Production decrease record exceeds the remaining capacity
        self.inventory_manager.remove_remaining_productivity("Gala", 100, 1)
        printed_output3 = mock_stdout.getvalue().strip()
        self.assertIn("The estimated yield will be set to 0", printed_output3,
                      "remove_remaining_productivity abnormal when production decrease")

        # remaining productivity not initialized
        self.inventory_manager.remaining_productivity = pd.read_csv("remaining_productivity.csv", nrows=0)
        self.inventory_manager.remaining_productivity.to_csv("remaining_productivity.csv", index=False)
        self.inventory_manager.remove_remaining_productivity("Ambrosia", 500)
        printed_output4 = mock_stdout.getvalue().strip()
        self.assertIn("Please estimate capacity first.", printed_output4,
                      "remove_remaining_productivity abnormal when data not initialized")

    @patch('sys.stdout', new_callable=StringIO)
    def test_get_current_inventory(self, mock_stdout):
        # normal condition
        self.assertListEqual(self.inventory_manager.get_current_inventory(), [2, 300, 0, 0, 0, 0, 0, 0, 400, 0],
                             "get_current_inventory abnormal")

        # empty inventory
        self.inventory_manager.inventory = pd.read_csv("inventory.csv", nrows=0)
        self.inventory_manager.inventory.to_csv("inventory.csv", index=False)
        self.inventory_manager.get_current_inventory()
        printed_output = mock_stdout.getvalue().strip()
        self.assertIn("Inventory is empty.", printed_output, "get_current_inventory abnormal when empty")

    def test_get_remaining_productivity(self):
        self.assertListEqual(self.inventory_manager.get_remaining_productivity(),
                             [1, 30000, 0, 0, 40000, 0, 0, 0, 72000, 0], "get_remaining_productivity abnormal")

    def test_get_extra_productivity(self):
        self.assertListEqual(self.inventory_manager.get_extra_productivity(), [1, 0, 100, 50, 0, 0, 0, 0, 0, 0],
                             "get_extra_productivity abnormal")


class TestHistoricalPlotter(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        print("Setting up class...")
        # Initialize common data for the entire test class
        cls.inventory_origin_df = pd.read_csv("management_testing_files/inventory_origin.csv")
        cls.remaining_productivity_origin_df = pd.read_csv(
            "management_testing_files/remaining_productivity_origin.csv")

    @classmethod
    def tearDownClass(cls):
        print("Tearing down class...")
        # Clean up any resources for entire class
        cls.inventory_origin_df = None
        cls.remaining_productivity_origin_df = None

    def setUp(self):
        print("Setting up test...")
        # Create a temporary DataFrame for testing purposes
        self.inventory_origin_df.to_csv("inventory.csv", index=False)
        self.remaining_productivity_origin_df.to_csv("remaining_productivity.csv", index=False)
        self.plotter = HistoricalPlotter("remaining_productivity.csv", "inventory.csv")

    def tearDown(self):
        print("Tearing down test...")
        # Clean up any resources used during the test
        del self.plotter
        files_to_remove = ["inventory.csv", "remaining_productivity.csv"]
        for file_name in files_to_remove:
            if os.path.exists(file_name):
                os.remove(file_name)

    def test_plot_historical_data(self):
        result = self.plotter.plot_historical_data(3, 1)
        self.assertEqual("Inventory Management - Peach", result, "plot_historical_data abnormal")
        with patch('pandas.read_csv') as mock_read_csv, patch('matplotlib.pyplot.show'):
            self.plotter.plot_historical_data(1, 1)
        mock_read_csv.assert_called_once_with("inventory.csv")
        with patch('pandas.read_csv') as mock_read_csv, patch('matplotlib.pyplot.show'):
            self.plotter.plot_historical_data(2, 2)
        mock_read_csv.assert_called_once_with("remaining_productivity.csv")


if __name__ == '__main__':
    unittest.main()



import os
import sys
import unittest
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from unittest_modules.fruit_unittest import FruitTestCase
from unittest_modules.fruit_unittest import FruitInfoTestCase
from unittest_modules.fruit_unittest import FruitInfoAddRemoveTestCase
from unittest_modules.plantation_unittest import RegionTestCase
from unittest_modules.unittest_inventory import TestInventoryManagement
from unittest_modules.unittest_inventory import TestHistoricalPlotter
from unittest_modules.unittest_sales import TestSalesManagement


def my_suite():
    suite = unittest.TestSuite()
    result = unittest.TestResult()

    suite.addTest(unittest.makeSuite(FruitTestCase))
    suite.addTest(unittest.makeSuite(FruitInfoTestCase))
    suite.addTest(unittest.makeSuite(FruitInfoAddRemoveTestCase))
    suite.addTest(unittest.makeSuite(RegionTestCase))
    suite.addTest(unittest.makeSuite(TestInventoryManagement))
    suite.addTest(unittest.makeSuite(TestHistoricalPlotter))
    suite.addTest(unittest.makeSuite(TestSalesManagement))

    runner = unittest.TextTestRunner()
    print(runner.run(suite))
if __name__ == '__main__':
    my_suite()




import os
import sys
from io import StringIO
import pandas as pd

import src.orchardmanagementpackage.production.fruit_info as fruit_info
import src.orchardmanagementpackage.production.plantation as plantation
import src.orchardmanagementpackage.management.inventory as inventory
import src.orchardmanagementpackage.management.sales as sales
import src.orchardmanagementpackage.fruit_info_operation as fruit_info_operation
import src.orchardmanagementpackage.plantation_operation as plantation_operation
import src.orchardmanagementpackage.inventory_operation as inventory_operation
import src.orchardmanagementpackage.sales_operation as sales_operation


def basic_choice_select(fruit_file, plantation_file, extra_productivity_file, remaining_productivity_file,
                        inventory_file, order_file):
    """
    Interact contents
    """
    while True:
        print("1. View and manage fruit categories in the orchard.")
        print("2. View and manage production regions in the orchard")
        print("3. Manage or visualize inventory of the orchard")
        print("4. Manage or visualize sales of the orchard")
        print("0. exit the system")

        choice_input = input("Please enter your choice: ")
        try:
            choice_input = int(choice_input)
            if choice_input == 1:
                fruit_info_operation.fruits_operations(fruit_file)
            elif choice_input == 2:
                plantation_operation.region_operations(fruit_file, plantation_file)

            elif choice_input == 3:
                inventory_operation.inventory_operator(extra_productivity_file, remaining_productivity_file,
                                                       inventory_file)
            elif choice_input == 4:
                sales_operation.sales_operator(extra_productivity_file, remaining_productivity_file, inventory_file, order_file)
            elif choice_input == 0:
                sys.exit()
            else:
                print("Invalid input, please enter a valid choice.")
        except ValueError:
            print("Invalid input, please enter the number of your choice.")


def file_import(relative_path):
    """
    Create absolute path
    """
    current_directory = os.getcwd()
    fruit_file = os.path.join(current_directory, relative_path)
    return fruit_file


def clear_csv_file(file_path):
    """
    Initialize files
    """
    # Read the CSV file, keeping only the column names
    df = pd.read_csv(file_path, nrows=0)
    # Overwrite the file, writing only the column names
    df.to_csv(file_path, index=False)


def fruit_init(fruit_file):
    '''
    Initialize fruit_information
    '''
    df = pd.DataFrame()
    df.to_csv(fruit_file, index=False)
    fruit_info.add_fruit(1, "Ambrosia", "big", "very sweet",
                         "less sour", "crunchy", 1.2, "pie", fruit_file)
    fruit_info.add_fruit(3, "Elberta", "big", "very sweet",
                         "less sour", "crunchy", 1.2, "pie", fruit_file)
    fruit_info.add_fruit(2, "Lapins", "small", "median sweet",
                         "median sour", "soft", 2.99, "cans", fruit_file)

def plantation_init(fruit_file, plantation_file):

    '''
    Initialize plantation information
    '''
    clear_csv_file(plantation_file)
    fruit_list = fruit_info.fruit_class_load(fruit_file)
    try:
        plantation.add_region(1, 1, "Ambrosia", 20, "market",
                              fruit_list, plantation_file)
        plantation.add_region(2, 3, "Elberta", 30, "market",
                              fruit_list, plantation_file)
        plantation.add_region(3, 2, "Lapins", 40, "pick",
                              fruit_list, plantation_file)
        plantation.add_region(4, 2, "Lapins", 40, "pick",
                              fruit_list, plantation_file)
    except plantation.FruitNoneException as e:
        print(f"Fail to add region, {e}")
    except plantation.RegionExistException as r:
        print(f"Fail to add region, {r}")



def inventory_init(extra_productivity_file, remaining_productivity_file, inventory_file):
    """
    Initialize inventory module
    """
    original_stdout = sys.stdout
    sys.stdout = StringIO()
    clear_csv_file(inventory_file)
    clear_csv_file(remaining_productivity_file)
    clear_csv_file(extra_productivity_file)
    inventory_manager = inventory.InventoryManagement(extra_productivity_file, remaining_productivity_file,
                                                      inventory_file)
    inventory_manager.total_product_estimate()
    inventory_manager.add_inventory('Ambrosia', 300)
    inventory_manager.add_inventory("Honeycrisp", 300)
    inventory_manager.add_inventory("Gala", 300)
    inventory_manager.add_inventory('Elberta', 400)
    inventory_manager.add_inventory('Lapins', 500)
    sys.stdout = original_stdout


def sales_init(extra_productivity_file, remaining_productivity_file, inventory_file, order_file):
    """
    Initialize sales module
    """
    original_stdout = sys.stdout
    sys.stdout = StringIO()
    inventory_manager = inventory.InventoryManagement(extra_productivity_file, remaining_productivity_file,
                                                      inventory_file, )
    clear_csv_file(order_file)
    # Instantiate the SalesManagement class with the inventory manager
    sales_manager = sales.SalesManagement(inventory_manager, order_file)
    # Add some orders
    sales_manager.add_order('selling', 'Ambrosia', 50)
    sales_manager.add_order('selling', 'Elberta', 50)
    sales_manager.add_order('picking', 'Lapins', 40)
    sys.stdout = original_stdout

def orchard_start():
    print("Welcome to the Orchard Management Software, where you can easily manage your orchard, record harvests, "
          "and create a fruitful orchard experience!")

    global fruit_file
    fruit_info_relative_path = "production/fruits.csv"
    fruit_file = file_import(fruit_info_relative_path)

    global plantation_file
    plantation_relative_path = "production/plantations.csv"
    plantation_file = file_import(plantation_relative_path)

    global extra_productivity_file
    extra_productivity_relative_path = "management/extra_productivity.csv"
    extra_productivity_file = file_import(extra_productivity_relative_path)

    global remaining_productivity_file
    remaining_productivity_path = "management/remaining_productivity.csv"
    remaining_productivity_file = file_import(remaining_productivity_path)

    global inventory_file
    inventory_file_path = "management/inventory.csv"
    inventory_file = file_import(inventory_file_path)

    global order_file
    order_file_path = "management/orders.csv"
    order_file = file_import(order_file_path)

    fruit_init(fruit_file)
    plantation_init(fruit_file, plantation_file)
    inventory_init(extra_productivity_file, remaining_productivity_file, inventory_file)
    sales_init(extra_productivity_file, remaining_productivity_file, inventory_file, order_file)
    basic_choice_select(fruit_file, plantation_file, extra_productivity_file, remaining_productivity_file,
                        inventory_file, order_file)

if __name__ == '__main__':
    orchard_start()




import codecs
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()

VERSION = '0.0.11'
DESCRIPTION = 'orchard management system'
LONG_DESCRIPTION = ('This repository contains the code for an Orchard Management Package, '
                    'designed to manage fruit production, inventory, and sales. '
                    'The system is implemented in Python and utilizes the pandas library for data manipulation and matplotlib for plotting.')

setup(
    name="orchardmanagementCL",
    version=VERSION,
    author="Yahan Cong, Kang Lu",
    author_email="",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    install_requires=[],
    keywords=['python', 'orchard', 'orchardmanagement'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows",
    ]
)



import src.orchardmanagementpackage.production.fruit_info as fruit_info
from datetime import datetime


def fruits_operations(fruit_file):
    """
    Provides operations related to fruit features and seasonal fruit selection.

    1. View fruit features: Display information about various fruit features.
    2. Check seasonal fruit selection: Check which fruits are in season.
    0. Back to main menu: Return to the main menu.

    Usage:
    fruits_operations(fruit_file)
    - 'fruit_file': Path to the CSV file containing fruit information.
    """

    # load fruit_list
    fruits_list = fruit_info.fruit_class_load(fruit_file)

    while True:
        print("Fruit operations: ")
        print("1. View fruit features")
        print("2. Check seasonal fruit selection")
        print("0. Back to main menu")
        fruit_choice_input = input("Please enter operation index here: ")

        try:
            fruit_choice_input = int(fruit_choice_input)
            if fruit_choice_input == 1:
                fruit_check(fruits_list)
            elif fruit_choice_input == 2:
                fruit_seasonal_check(fruits_list)
            elif fruit_choice_input == 0:
                print("Welcome to the Orchard Management Software")
                break
            else:
                print("Invalid operation index")
        except ValueError:
            print("Invalid input, please enter operation index.")


def fruit_check(fruits_list):
    """
    Provides functionality to check and display information about different fruit types.
    """
    apple_list = []
    cherry_list = []
    peach_list = []

    apple_variety = set()
    cherry_variety = set()
    peach_variety = set()

    for fruit in fruits_list:
        if isinstance(fruit, fruit_info.Apple):
            apple_list.append(fruit)
            apple_variety.add(fruit.variety)
        elif isinstance(fruit, fruit_info.Cherry):
            cherry_list.append(fruit)
            cherry_variety.add(fruit.variety)
        else:
            peach_list.append(fruit)
            peach_variety.add(fruit.variety)

    num_input = input("Please enter the index of the fruit you want to search for: 1. Apple, 2. Cherry, 3. Peach :  ")
    try:
        num_input = int(num_input)
        if num_input == 1:
            fruit_variety_check(num_input, apple_variety, fruits_list, "apple")
        elif num_input == 2:
            fruit_variety_check(num_input, cherry_variety, fruits_list, "cherry")
        elif num_input == 3:
            fruit_variety_check(num_input, peach_variety, fruits_list, "peach")
        else:
            print("Invalid fruit index.")
    except ValueError:
        print("Please enter a valid fruit index")


def fruit_variety_check(fruit_type_num, variety_set, fruits_list, fruit_name):
    """
    Displays information about a specific variety of a fruit type.
    """
    variety_input = input(f"Please enter a {fruit_name} variety: {variety_set}:  ")
    curr_date = int(datetime.now().date().strftime("%m"))

    if isinstance(variety_input, str) and variety_input in variety_set:
        fruit_searched = fruit_info.get_fruit(fruit_type_num, variety_input, fruits_list)
        print(f"{variety_input} {fruit_name}: {fruit_searched.describe()}")
        fruit_searched.get_available_season(curr_date)
    else:
        print("Invalid input")


def fruit_seasonal_check(fruits_list):
    """
    Checks and displays available fruits for the current date or a specific date.
    """
    curr_time = datetime.now().date().strftime("%m-%d")
    available_list = fruit_info.available_season_fruit(curr_time, fruits_list)
    if not available_list:
        print(f"Sorry, at date {curr_time}, there is no fruit available now")
    else:
        print(f"Available fruits at {curr_time}: ")
        for fruit_available in available_list:
            print(f"{fruit_available.variety} {fruit_available.get_type()}")
    print("Please enter the date if you want to check seasonal fruit for a specific time. "
          "or enter 'back' to return to the previous menu.")

    date_input = input("Enter datetime with format 'MM-DD' or 'back': ")
    try:
        if date_input == "back":
            fruits_operations()
        date_specific_list = fruit_info.available_season_fruit(date_input, fruits_list)
        if not date_specific_list:
            print(f"Sorry, there is no fruit available on {date_input}")
        else:
            print(f"Available fruits at {date_input}: ")
            for fruit_available in date_specific_list:
                print(f"{fruit_available.variety} {fruit_available.get_type()}")
    except:
        print("Invalid date")